
from textual.app import ComposeResult
from textual.containers import VerticalScroll
from textual.widgets import DataTable, Label

from blackwall.panels.search.search_backend import QueryType

USER_COLUMNS = [
    ("User", "Owner", "dfltgrp", "SOA", "RIRP", "UID", "Shell", "Home", "Last logon", "Created"),
]

GROUP_COLUMNS = [
    ("Group", "Connected users", "Created"),
]

DATASET_COLUMNS = [
    ("Dataset", "UACC", "Owner", "Created"),
]

class PanelResultsUsers(VerticalScroll):
    def __init__(self, user_dict: dict):
        super().__init__()
        self.user_dict = user_dict

    def compose(self) -> ComposeResult:
        yield Label("Users:")
        yield DataTable(id="results_user_table")     

    def on_mount(self) -> None:
        user_table = self.get_child_by_id("results_user_table",DataTable)
        user_table.zebra_stripes = True
        user_table.add_columns(*USER_COLUMNS[0]) 

class PanelResultsGroup(VerticalScroll):
    def __init__(self, group_dict: dict):
        super().__init__()
        self.group_dict = group_dict

    def compose(self) -> ComposeResult:
        yield Label("Groups:")
        yield DataTable(id="results_group_table")    

    def on_mount(self) -> None:
        group_table = self.get_child_by_id("results_group_table",DataTable)
        group_table.zebra_stripes = True
        group_table.add_columns(*GROUP_COLUMNS[0])

class PanelResultsDatasets(VerticalScroll):
    def __init__(self, dataset_dict: dict):
        super().__init__()
        self.dataset_dict = dataset_dict

    def compose(self) -> ComposeResult:
        yield Label("Dataset profiles:")
        yield DataTable(id="results_dataset_table")        

    def on_mount(self) -> None:
        dataset_table = self.get_child_by_id("results_dataset_table",DataTable)
        dataset_table.zebra_stripes = True
        dataset_table.add_columns(*DATASET_COLUMNS[0])

class PanelResultsResources(VerticalScroll):
    def __init__(self, resource_dict: dict):
        super().__init__()
        self.resource_dict = resource_dict

    def compose(self) -> ComposeResult:
        yield Label("General resources profiles:")
        yield DataTable(id="results_dataset_table")        

class PanelResultsMixedType(VerticalScroll):
    def __init__(self, results: dict[QueryType,dict]):
        super().__init__()
        self.results = results

    def compose(self) -> ComposeResult:
        if QueryType.User in self.results:
            yield PanelResultsUsers(user_dict=self.results[QueryType.User])
        if QueryType.Group in self.results:
            yield PanelResultsGroup(group_dict=self.results[QueryType.Group])
        if QueryType.Dataset in self.results:
            yield PanelResultsDatasets(dataset_dict=self.results[QueryType.Dataset])
        if QueryType.Resource in self.results:
            yield PanelResultsResources(resource_dict=self.results[QueryType.Resource])



from textual.app import ComposeResult
from textual.containers import VerticalScroll
from textual.widgets import Label, OptionList
from textual.widgets.option_list import Option


class PanelBackout(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield Label("Backout changes",classes="backout-title")
        yield OptionList(
            Option("Create user 27/04/2025 at 20:30"),
            None,
            Option("Delete user 27/04/2025 at 20:30"),
            None,
            Option("Create resource profile 27/04/2025 at 20:30"),
            None,
            Option("Update dataset profile 27/04/2025 at 20:30"),
        )



from textual.app import ComposeResult
from textual.containers import Grid
from textual.screen import Screen
from textual.widget import Widget
from textual.widgets import Button, Label


class ConfirmationScreen(Screen):
    """Modal confirmation screen"""

    def __init__(self, dialog_text: str, confirm_action: str,action_widget: Widget):
        super().__init__()
        self.dialog_text = dialog_text
        self.confirm_action = confirm_action
        self.action_widget = action_widget

    def compose(self) -> ComposeResult:
        yield Grid(
            Label(self.dialog_text, id="question"),
            Button("Cancel", variant="primary", id="cancel", classes="modal-buttons"),
            Button("Confirm", variant="error", id="confirm", classes="modal-buttons"),
            id="dialog",
        )

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "confirm":
            self.app.pop_screen()
            await self.app.run_action(self.confirm_action,default_namespace=self.action_widget)
        else:
            self.app.pop_screen()



#User API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

import importlib.util
from dataclasses import dataclass, field
from typing import Any

from .traits_base import TraitsBase

#Checks if RACFU can be imported
racfu_enabled = importlib.util.find_spec('racfu')

if racfu_enabled:
    from racfu import racfu  # type: ignore
else:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")       

@dataclass
class BaseUserTraits(TraitsBase):
    #primary
    owner: str | None = field(default=None,metadata={"label": "Owner", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    default_group: str | None = field(default=None,metadata={"label": "Default group", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    name: str | None = field(default=None,metadata={"label": "Name", "input_args": {"max_length": 20,"classes": "field-medium-generic"}})
    installation_data: str | None = field(default=None,metadata={"label": "Installation data", "input_args": {"max_length": 255,"classes": "field-long-generic"}})

    #user attributes
    special: bool | None = field(default=None,metadata={"label": "Special"})
    operations: bool | None = field(default=None,metadata={"label": "Operations"})
    auditor: bool | None = field(default=None,metadata={"label": "Auditor"})

    password: str | None = field(default=None, metadata={
        "masked": True,
        "maximum": 8,
    })
    
    passphrase: str | None = field(default=None, metadata={
        "masked": True,
        "minimum": 12,
    })
    
    default_group_authority: str | None = field(default=None,metadata={"label": "Default group authority"})
    security_category: str | None = field(default=None,metadata={"label": "Security category"})
    security_level: str | None = field(default=None,metadata={"label": "Security level"})
    security_label: str | None = field(default=None,metadata={"label": "Security label"})
    class_authorization: str | None = field(default=None)
    universal_access: str | None = field(default=None,metadata={"label": "UACC"})

    model_data_set: str | None = field(default=None,metadata={"label": "Model dataset"})
    group_data_set_access: bool | None = field(default=None,metadata={"label": "Group dataset access"})

    create_date: str | None = field(default=None,metadata={"label": "Create date","allowed_in": {"extract"}})
    last_access_date: str | None = field(default=None,metadata={"label": "Last access date","allowed_in": {"extract"}})
    last_access_time: str | None = field(default=None,metadata={"label": "Last access time","allowed_in": {"extract"}})

@dataclass
class CICSUserTraits(TraitsBase):
    operator_class: str | None = field(default=None, metadata={"label": "operator class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    operator_id: str | None = field(default=None, metadata={"label": "operator id", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    operator_priority: str | None = field(default=None, metadata={"label": "operator priority", "input_args": {"max_length": 4,"classes": "field-short-generic"}})
    resource_security_level_key: str | None = field(default=None, metadata={"label": "resource security level key", "input_args": {"classes": "field-long-generic"}})
    resource_security_level_keys: str | None = field(default=None, metadata={"label": "resource security level keys", "input_args": {"classes": "field-long-generic"}})
    timeout: str | None = field(default=None, metadata={"label": "timeout", "input_args": {"max_length": 4,"classes": "field-short-generic"}})
    transaction_security_level_key: str | None = field(default=None, metadata={"label": "transaction security level key", "input_args": {"classes": "field-long-generic"}})
    force_signoff_when_xrf_takeover: bool | None = field(default=None, metadata={"label": "Force signoff when xrf takeover", "input_args": {"classes": "field-long-generic"}})

@dataclass
class DCEUserTraits(TraitsBase):
    auto_login: bool | None = field(default=None, metadata={"label": "auto login"})
    name: str | None = None
    home_cell: str | None = None
    home_cell_uuid: str | None = None
    uuid: str | None = None

@dataclass
class DFPUserTraits(TraitsBase):
    data_application: str | None = field(default=None, metadata={"label": "data application", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    data_class: str | None = field(default=None, metadata={"label": "data class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    management_class: str | None = field(default=None, metadata={"label": "management class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    storage_class: str | None = field(default=None, metadata={"label": "storage class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})

@dataclass
class EIMUserTraits(TraitsBase):
    ldap_bind_profile: str | None = field(default=None, metadata={"label": "LDAP bind profile", "input_args": {"max_length": 255,"classes": "field-long-generic"}})

@dataclass
class KerbUserTraits(TraitsBase):
    encryption_algorithm: str | None = field(default=None, metadata={"label": "encryption algorithm", "input_args": {"max_length": 255,"classes": "field-long-generic"}})
    name: str | None = field(default=None, metadata={"label": "name", "input_args": {"max_length": 255,"classes": "field-long-generic"}})
    key_from: str | None = field(default=None, metadata={"allowed_in": {"extract"}, "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    key_version: str | None = field(default=None, metadata={"allowed_in": {"extract"}, "input_args": {"max_length": 3,"classes": "field-short-generic"}})
    max_ticket_life: int | None = field(default=None, metadata={"label": "max ticket life", "input_args": {"max_length": 16,"classes": "field-medium-generic"}})

@dataclass
class LanguageUserTraits(TraitsBase):
    primary: str | None = field(default=None, metadata={"label": "primary language", "input_args": {"max_length": 4,"classes": "field-short-generic"}})
    secondary: str | None = field(default=None, metadata={"label": "secondary language", "input_args": {"max_length": 4,"classes": "field-short-generic"}})

@dataclass
class LnotesUserTraits(TraitsBase):
    zos_short_name: str | None = field(default=None, metadata={"label": "Lotus Notes short name", "input_args": {"max_length": 64,"classes": "field-medium-generic"}})

@dataclass
class MfaUserTraits(TraitsBase):
    factor: str | None = field(default=None, metadata={"label": "factor", "allowed_in": {"alter","extract"}})
    active: bool | None = field(default=None, metadata={"label": "mfa active", "allowed_in": {"alter","extract"}})
    tags: str | None = field(default=None, metadata={"label": "tags", "allowed_in": {"alter"}})
    password_fallback: bool | None = field(default=None, metadata={"label": "password fallback", "allowed_in": {"alter","extract"}})
    mfa_policy: str | None = field(default=None, metadata={"label": "MFA policy", "allowed_in": {"alter","extract"}})

@dataclass
class NDSUserTraits(TraitsBase):
    username: str | None = field(default=None, metadata={"label": "username", "input_args": {"max_length": 255,"classes": "field-long-generic"}})

@dataclass
class NetviewUserTraits(TraitsBase):
    default_mcs_console_name: str | None = None
    security_control_check: str | None = None
    domain: str | None = None
    logon_commands: str | None = None
    receive_unsolicited_messages: str | None = None
    operator_graphic_monitor_facility_administration_allowed: str | None = None
    operator_graphic_monitor_facility_display_authority: str | None = None
    operator_scope_classes: str | None = None

@dataclass
class OMVSUserTraits(TraitsBase):
    uid: int | None = field(default=None, metadata={"label": "uid", "input_args": {"classes": "field-long-generic"}})
    auto_uid: bool | None = field(default=None, metadata={"label": "auto uid", "invalid_values": {False}})
    shared: bool | None = field(default=None, metadata={"label": "shared", "invalid_values": {False}})
    home_directory: str | None = field(default=None, metadata={"label": "home directory", "input_args": {"classes": "field-long-generic"}})
    default_shell: str | None = field(default=None, metadata={"label": "default shell", "input_args": {"classes": "field-long-generic"}})
    max_address_space_size: int | None = field(default=None, metadata={"label": "max address space size", "input_args": {"classes": "field-long-generic"}})
    max_cpu_time: int | None = field(default=None, metadata={"label": "home directory", "input_args": {"max_length": 10, "classes": "field-medium-generic"}})
    max_files_per_process: int | None = field(default=None, metadata={"label": "max files per process", "input_args": {"classes": "field-long-generic"}})
    max_file_mapping_pages: int | None = field(default=None, metadata={"label": "max file mapping pages", "input_args": {"classes": "field-long-generic"}})
    max_processes: int | None = field(default=None, metadata={"label": "max processes", "input_args": {"classes": "field-long-generic"}})
    max_shared_memory: int | None = field(default=None, metadata={"label": "max shared memory", "input_args": {"max_length": 10, "classes": "field-medium-generic"}})
    max_non_shared_memory: str | None = field(default=None, metadata={"label": "max non shared memory", "input_args": {"max_length": 10, "classes": "field-medium-generic"}})
    max_threads: int | None = field(default=None, metadata={"label": "max threads", "input_args": {"max_length": 10, "classes": "field-medium-generic"}})

@dataclass
class OperparmUserTraits(TraitsBase):
    alternate_console_group: str | None = field(default=None, metadata={"label": "alternate console group", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    receive_automated_messages: str | None = None
    command_target_system: str | None = None
    receive_delete_operator_messages: str | None = field(default=None, metadata={"label": "receive delete operator messages", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    receive_hardcopy_messages: str | None = None
    receive_internal_console_messages: str | None = None
    console_searching_key: str | None = None
    message_level: str | None = None
    log_command_responses: str | None = None
    message_format: str | None = None
    migration_id: str | None = None
    monitor_event: str | None = None
    message_scope: str | None = None
    console_authority: str | None = None
    receive_routing_code: str | None = None
    message_queue_storage: str | None = None
    receive_undelivered_messages: str | None = None
    receive_unknown_console_id_messages: str | None = None

@dataclass
class OvmUserTraits(TraitsBase):
    file_system_root: str | None = field(default=None, metadata={"label": "file system root", "input_args": {"classes": "field-long-generic"}})
    home_directory: str | None = field(default=None, metadata={"label": "home directory", "input_args": {"classes": "field-long-generic"}})
    default_shell: str | None = field(default=None, metadata={"label": "default shell", "input_args": {"classes": "field-long-generic"}})
    uid: str | None = field(default=None, metadata={"label": "uid", "input_args": {"classes": "field-long-generic"}})

@dataclass
class ProxyUserTraits(TraitsBase):
    bind_distinguished_name: str | None = field(default=None, metadata={"label": "bind distinguished name"})
    bind_password: str | None = field(default=None, metadata={"label": "bind password","input_args": {"password": True}})
    ldap_host: str | None = field(default=None, metadata={"label": "LDAP host"})

@dataclass
class TSOUserTraits(TraitsBase):
    account_number: str | None = field(default=None, metadata={"label": "account number", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    logon_command: str | None = field(default=None, metadata={"label": "logon command", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    sysout_destination_id: str | None = field(default=None, metadata={"label": "sysout destination id", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    hold_class: str | None = field(default=None, metadata={"label": "hold class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    job_class: str | None = field(default=None, metadata={"label": "job class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    message_class: str | None = field(default=None, metadata={"label": "message class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    logon_procedure: str | None = field(default=None, metadata={"label": "logon procedure", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    security_label: str | None = field(default=None, metadata={"label": "security label", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    default_region_size: int | None = field(default=None, metadata={"label": "default region size", "input_args": {"classes": "field-medium-generic"}})
    max_region_size: int | None = field(default=None, metadata={"label": "max region size", "input_args": {"classes": "field-medium-generic"}})
    sysout_class: str | None = field(default=None, metadata={"label": "sysout class", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    data_set_allocation_unit: str | None = field(default=None, metadata={"label": "dataset allocation unit", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    user_data: str | None = field(default=None, metadata={"label": "user data", "input_args": {"max_length": 8,"classes": "field-short-generic"}})

@dataclass
class WorkattrUserTraits(TraitsBase):
    account_number: str | None = field(default=None, metadata={"label": "account number", "input_args": {"max_length": 255,"classes": "field-long-generic"}})
    sysout_building: str | None = field(default=None, metadata={"label": "sysout building", "input_args": {"max_length": 60, "classes": "field-medium-generic"}})
    sysout_department: str | None = field(default=None, metadata={"label": "sysout department", "input_args": {"max_length": 60, "classes": "field-medium-generic"}})
    sysout_user: str | None = field(default=None, metadata={"label": "sysout user", "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    sysout_room: str | None = field(default=None, metadata={"label": "sysout room", "input_args": {"max_length": 60, "classes": "field-medium-generic"}})
    sysout_email: str | None = field(default=None, metadata={"label": "email", "input_args": {"max_length": 255,"classes": "field-long-generic"}})


#User functions
def user_exists(username: str) -> bool:
    """Checks if a user exists, returns true or false"""
    if racfu_enabled:
        result = racfu({"operation": "extract", "admin_type": "user", "profile_name": username.upper()})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return False
    
def get_user(username: str) -> dict[str, Any]:
    """Doesn't handle users that don't exist, recommend using user_exists() first"""
    if racfu_enabled:
        result = racfu({"operation": "extract", "admin_type": "user", "profile_name": username.upper()})
        return result.result
    else:
        return False

def update_user(
        username: str, 
        create: bool,
        base: BaseUserTraits, 
        cics: CICSUserTraits | None = None, 
        dce: DCEUserTraits | None = None, 
        dfp: DFPUserTraits | None = None, 
        eim: EIMUserTraits | None = None, 
        language: LanguageUserTraits | None = None, 
        lnotes: LnotesUserTraits | None = None, 
        mfa: MfaUserTraits | None = None, 
        nds: NDSUserTraits | None = None, 
        netview: NetviewUserTraits | None = None, 
        omvs: OMVSUserTraits | None = None,
        operparm: OperparmUserTraits | None = None, 
        ovm: OvmUserTraits | None = None, 
        proxy: ProxyUserTraits | None = None, 
        tso: TSOUserTraits | None = None, 
        workattr: WorkattrUserTraits | None = None, 
        ):
    """Update or creates a new user, returns true if the user was successfully created and false if an error code was given"""
    traits = base.to_traits(prefix="base")
    
    if cics is not None:
        traits.update(cics.to_traits("cics"))

    if dce is not None:
        traits.update(dce.to_traits("dce"))

    if dfp is not None:
        traits.update(dfp.to_traits("dfp"))

    if eim is not None:
        traits.update(eim.to_traits("eim"))

    if language is not None:
        traits.update(language.to_traits("language"))

    if lnotes is not None:
        traits.update(lnotes.to_traits("lnotes"))

    if mfa is not None:
        traits.update(mfa.to_traits("mfa"))

    if nds is not None:
        traits.update(nds.to_traits("nds"))

    if netview is not None:
        traits.update(netview.to_traits("netview"))

    if omvs is not None:
        traits.update(omvs.to_traits("omvs"))

    if operparm is not None:
        traits.update(operparm.to_traits("operparm"))

    if ovm is not None:
        traits.update(ovm.to_traits("ovm"))

    if proxy is not None:
        traits.update(proxy.to_traits("proxy"))

    if tso is not None:
        traits.update(tso.to_traits("tso"))
    
    if workattr is not None:
        traits.update(workattr.to_traits("workattr"))

    operation = "add" if create else "alter"

    result = racfu(
            {
                "operation": operation, 
                "admin_type": "user", 
                "profile_name": username,
                "traits":  traits,
            },
        )
    return result.result["return_codes"]["racf_return_code"]

def delete_user(username: str) -> tuple[str, int]:
    if racfu_enabled:
        """Deletes a user"""
        result = racfu(
                {
                    "operation": "delete", 
                    "admin_type": "user", 
                    "profile_name": username.upper(),
                },
            )
        return result.result["commands"][0]["messages"][0], result.result["return_codes"]["racf_return_code"]
    else:
        return "RACFu can't be found", 8




from textual.widget import Widget

from blackwall.screens.modal.confirmation import ConfirmationScreen
from blackwall.screens.modal.warning import WarningScreen


def generic_confirmation_modal(self: Widget, modal_text: str, confirm_action: str, action_widget: Widget) -> None:
    modal_screen = ConfirmationScreen(dialog_text=modal_text,confirm_action=confirm_action,action_widget=action_widget)

    self.app.push_screen(modal_screen)

def generic_warning_modal(self: Widget, modal_text: str) -> None:
    modal_screen = WarningScreen(dialog_text=modal_text)

    self.app.push_screen(modal_screen)


import tomllib
from pathlib import Path

default_user_config_name = ".config/blackwall/user_config.toml"
default_site_config_name = "/etc/blackwall/site_config.toml"

site_config_path = Path(default_site_config_name)

if site_config_path.exists():
    site_config = site_config_path.open().read()
    site_settings = tomllib.loads(site_config)
else:
    site_settings = None

user_config_path = Path.home()/default_user_config_name

if user_config_path.exists():
    user_config = user_config_path.open().read()
    user_settings = tomllib.loads(user_config)
else:
    user_settings = None

def extract_setting(conf_dict: dict | None,section: str, setting: str):
    if conf_dict is not None:
        section_dict = conf_dict.get(section)
        if section_dict is not None:
            return section_dict.get(setting)
    return None

def get_site_setting(section: str,setting: str):
    #load settings from site config
    return extract_setting(conf_dict=site_settings,section=section,setting=setting)

def get_user_setting(section: str,setting: str):
    #load settings from user config
    return extract_setting(conf_dict=user_settings,section=section,setting=setting)




#TODO fix all of this shitty code. Not gonna lie the code below is really bad, but I was in a crunch

from textual import on
from textual.app import ComposeResult
from textual.containers import HorizontalGroup, VerticalScroll
from textual.widgets import Button, Input, Label, RadioButton, RadioSet

from blackwall.api import dataset, group, resource, user
from blackwall.messages import OpenTab
from blackwall.panels.dataset.dataset import DatasetInfo, PanelDataset
from blackwall.panels.group.group import GroupInfo, PanelGroup
from blackwall.panels.panel_mode import PanelMode
from blackwall.panels.resource.resource import PanelResource, ResourceInfo
from blackwall.panels.search.results import PanelResultsMixedType
from blackwall.panels.search.search_backend import QueryType, search_database_query_one
from blackwall.panels.users.user import PanelUser, UserInfo


class SearchSelector(HorizontalGroup):
    def compose(self) -> ComposeResult:
        with RadioSet(id="type_selector",classes="search-selector"):
            yield RadioButton("Any",id="search_type_any",value=False,disabled=True)
            yield RadioButton("User",id="search_type_user",value=True)
            yield RadioButton("Group",id="search_type_group")
            yield RadioButton("Dataset profile",id="search_type_dataset")
            yield RadioButton("Resource profile",id="search_type_resource")
        with RadioSet(id="filter-selector",classes="search-selector"):
            yield RadioButton("All",disabled=True)
            yield RadioButton("Only one",value=True)

class SearchField(HorizontalGroup):
    def __init__(self, search_action: str):
        super().__init__()
        self.search_action = search_action

    def compose(self) -> ComposeResult:
        yield Label("Search:")
        yield Input(name="Class",id="search_field_class",classes="field-short-generic")
        yield Input(name="Search",id="search_field",classes="search-field")
        yield Button("Search",action="search")

    async def action_search(self):
        await self.app.run_action(self.search_action,default_namespace=self.parent)

class PanelSearch(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield SearchSelector()
        yield SearchField(search_action="search")

    @on(Input.Submitted)
    def action_search(self) -> None:
        search_query = self.get_child_by_type(SearchField).get_child_by_id("search_field",Input).value
        search_query_class = self.get_child_by_type(SearchField).get_child_by_id("search_field_class",Input).value
        search_type = self.get_child_by_type(SearchSelector).get_child_by_id("type_selector",RadioSet).pressed_button.id # type: ignore
        if search_type == "search_type_any":
            results = search_database_query_one(query=search_query, class_name=None,query_types=QueryType.all())
            self.post_message(OpenTab("Results",PanelResultsMixedType(results)))
        elif search_type == "search_type_user":
            if user.user_exists(username=search_query):
                new_user_panel = PanelUser()

                user_dict = user.get_user(username=search_query)
            
                base_traits = user.BaseUserTraits.from_dict(prefix="base",source=user_dict["profile"]["base"])

                new_user_panel.user_info = UserInfo(
                    base_traits=base_traits,
                    username=search_query,
                    mode=PanelMode.edit,
                )

                if 'profile' in user_dict and 'tso' in user_dict['profile']:
                    new_user_panel.user_info.tso_traits = user.TSOUserTraits.from_dict(prefix="tso",source=user_dict["profile"]["tso"])

                if 'profile' in user_dict and 'omvs' in user_dict['profile']:
                    new_user_panel.user_info.omvs_traits = user.OMVSUserTraits.from_dict(prefix="omvs",source=user_dict["profile"]["omvs"])

                if 'profile' in user_dict and 'nds' in user_dict['profile']:
                    new_user_panel.user_info.nds_traits = user.NDSUserTraits.from_dict(prefix="nds",source=user_dict["profile"]["nds"])

                if 'profile' in user_dict and 'cics' in user_dict['profile']:
                    new_user_panel.user_info.cics_traits = user.CICSUserTraits.from_dict(prefix="cics",source=user_dict["profile"]["cics"])

                if 'profile' in user_dict and 'netview' in user_dict['profile']:
                    new_user_panel.user_info.netview_traits = user.NetviewUserTraits.from_dict(prefix="netview",source=user_dict["profile"]["netview"])

                if 'profile' in user_dict and 'mfa' in user_dict['profile']:
                    new_user_panel.user_info.mfa_traits = user.MfaUserTraits.from_dict(prefix="mfa",source=user_dict["profile"]["mfa"])
                
                if 'profile' in user_dict and 'eim' in user_dict['profile']:
                    new_user_panel.user_info.eim_traits = user.EIMUserTraits.from_dict(prefix="eim",source=user_dict["profile"]["eim"])

                if 'profile' in user_dict and 'workattr' in user_dict['profile']:
                    new_user_panel.user_info.workattr_traits = user.WorkattrUserTraits.from_dict(prefix="workattr",source=user_dict["profile"]["workattr"])
                
                if 'profile' in user_dict and 'ovm' in user_dict['profile']:
                    new_user_panel.user_info.ovm_traits = user.OvmUserTraits.from_dict(prefix="ovm",source=user_dict["profile"]["ovm"])

                if 'profile' in user_dict and 'dce' in user_dict['profile']:
                    new_user_panel.user_info.dce_traits = user.DCEUserTraits.from_dict(prefix="dce",source=user_dict["profile"]["dce"])
                
                if 'profile' in user_dict and 'dfp' in user_dict['profile']:
                    new_user_panel.user_info.dfp_traits = user.DFPUserTraits.from_dict(prefix="dfp",source=user_dict["profile"]["dfp"])

                if 'profile' in user_dict and 'operparm' in user_dict['profile']:
                    new_user_panel.user_info.operparm_traits = user.OperparmUserTraits.from_dict(prefix="operparm",source=user_dict["profile"]["operparm"])

                if 'profile' in user_dict and 'proxy' in user_dict['profile']:
                    new_user_panel.user_info.proxy_traits = user.ProxyUserTraits.from_dict(prefix="proxy",source=user_dict["profile"]["proxy"])
                
                if 'profile' in user_dict and 'lnotes' in user_dict['profile']:
                    new_user_panel.user_info.lnotes_traits = user.LnotesUserTraits.from_dict(prefix="lnotes",source=user_dict["profile"]["lnotes"])

                if 'profile' in user_dict and 'language' in user_dict['profile']:
                    new_user_panel.user_info.lang_traits = user.LanguageUserTraits.from_dict(prefix="language",source=user_dict["profile"]["language"])

                if 'profile' in user_dict and 'kerb' in user_dict['profile']:
                    new_user_panel.user_info.kerb_traits = user.KerbUserTraits.from_dict(prefix="kerb",source=user_dict["profile"]["kerb"])
                
                self.post_message(OpenTab(f"User: {search_query}",new_user_panel))

                self.notify(f"Found user: {search_query}")
            else:
                self.notify(f"User {search_query} couldn't be found")
        elif search_type == "search_type_group":
            if group.group_exists(group=search_query):
                new_group_panel = PanelGroup()

                group_dict = group.get_group(group=search_query)
            
                base_traits = group.BaseGroupTraits.from_dict(prefix="base",source=group_dict["profile"]["base"])

                new_group_panel.group_info = GroupInfo(
                    base_traits=base_traits,
                    group_name=search_query,
                )

                if 'profile' in group_dict and 'dfp' in group_dict['profile']:
                    new_group_panel.group_info.dfp_traits = group.DFPGroupTraits.from_dict(prefix="dfp",source=group_dict["profile"]["dfp"])

                self.post_message(OpenTab(f"Group: {search_query}",new_group_panel))

                self.notify(f"Found group: {search_query}")
            else:
                self.notify(f"Group {search_query} couldn't be found")
        elif search_type == "search_type_dataset":
            if dataset.dataset_profile_exists(dataset=search_query):
                new_dataset_panel = PanelDataset()

                dataset_dict = dataset.get_dataset_profile(dataset=search_query)
            
                base_traits = dataset.BaseDatasetTraits.from_dict(prefix="base",source=dataset_dict["profile"]["base"])

                new_dataset_panel.dataset_info = DatasetInfo(
                    base_traits=base_traits,
                    profile_name=search_query,
                )

                self.post_message(OpenTab(f"Dataset: {search_query}",new_dataset_panel))

                self.notify(f"Found dataset profile: {search_query}")
            else:
                self.notify(f"Dataset profile {search_query} couldn't be found")
        elif search_type == "search_type_resource":
            if resource.resource_profile_exists(resource=search_query,resource_class=search_query_class):
                new_resource_panel = PanelResource()

                resource_dict = resource.get_resource_profile(resource=search_query,resource_class=search_query_class)
            
                base_traits = resource.BaseResourceTraits.from_dict(prefix="base",source=resource_dict["profile"]["base"])

                new_resource_panel.resource_info = ResourceInfo(
                    base_traits=base_traits,
                    resource_class=search_query_class,
                    resource_name=search_query,
                )

                if 'profile' in resource_dict and 'kerb' in resource_dict['profile']:
                    new_resource_panel.resource_info.kerb_traits = resource.KerbResourceTraits.from_dict(prefix="kerb",source=resource_dict["profile"]["kerb"])

                if 'profile' in resource_dict and 'dlf' in resource_dict['profile']:
                    new_resource_panel.resource_info.dlf_traits = resource.DLFDataResourceTraits.from_dict(prefix="dlf",source=resource_dict["profile"]["dlf"])

                if 'profile' in resource_dict and 'eim' in resource_dict['profile']:
                    new_resource_panel.resource_info.eim_traits = resource.EIMResourceTraits.from_dict(prefix="eim",source=resource_dict["profile"]["eim"])

                if 'profile' in resource_dict and 'jes' in resource_dict['profile']:
                    new_resource_panel.resource_info.jes_traits = resource.JESResourceTraits.from_dict(prefix="jes",source=resource_dict["profile"]["jes"])

                if 'profile' in resource_dict and 'icsf' in resource_dict['profile']:
                    new_resource_panel.resource_info.icsf_traits = resource.ICSFResourceTraits.from_dict(prefix="icsf",source=resource_dict["profile"]["icsf"])

                if 'profile' in resource_dict and 'kerb' in resource_dict['profile']:
                    new_resource_panel.resource_info.kerb_traits = resource.KerbResourceTraits.from_dict(prefix="kerb",source=resource_dict["profile"]["kerb"])

                if 'profile' in resource_dict and 'ictx' in resource_dict['profile']:
                    new_resource_panel.resource_info.ictx_traits = resource.ICTXResourceTraits.from_dict(prefix="ictx",source=resource_dict["profile"]["ictx"])

                if 'profile' in resource_dict and 'idtparms' in resource_dict['profile']:
                    new_resource_panel.resource_info.idtparms_traits = resource.IDTPARMSResourceTraits.from_dict(prefix="idtparms",source=resource_dict["profile"]["idtparms"])

                if 'profile' in resource_dict and 'session' in resource_dict['profile']:
                    new_resource_panel.resource_info.session_traits = resource.SessionResourceTraits.from_dict(prefix="session",source=resource_dict["profile"]["session"])

                if 'profile' in resource_dict and 'svfmr' in resource_dict['profile']:
                    new_resource_panel.resource_info.svfmr_traits = resource.SVFMRResourceTraits.from_dict(prefix="svfmr",source=resource_dict["profile"]["svfmr"])

                if 'profile' in resource_dict and 'stdata' in resource_dict['profile']:
                    new_resource_panel.resource_info.stdata_traits = resource.STDATAResourceTraits.from_dict(prefix="stdata",source=resource_dict["profile"]["stdata"])

                if 'profile' in resource_dict and 'proxy' in resource_dict['profile']:
                    new_resource_panel.resource_info.proxy_traits = resource.ProxyResourceTraits.from_dict(prefix="proxy",source=resource_dict["profile"]["proxy"])

                if 'profile' in resource_dict and 'mfpolicy' in resource_dict['profile']:
                    new_resource_panel.resource_info.mfpolicy_traits = resource.MFPolicyResourceTraits.from_dict(prefix="mfpolicy",source=resource_dict["profile"]["mfpolicy"])

                if 'profile' in resource_dict and 'sigver' in resource_dict['profile']:
                    new_resource_panel.resource_info.sigver_traits = resource.SIGVERResourceTraits.from_dict(prefix="sigver",source=resource_dict["profile"]["sigver"])

                if 'profile' in resource_dict and 'tme' in resource_dict['profile']:
                    new_resource_panel.resource_info.tme_traits = resource.TMEResourceTraits.from_dict(prefix="tme",source=resource_dict["profile"]["tme"])

                if 'profile' in resource_dict and 'cdtinfo' in resource_dict['profile']:
                    new_resource_panel.resource_info.cdtinfo_traits = resource.CDTINFOResourceTraits.from_dict(prefix="cdtinfo",source=resource_dict["profile"]["cdtinfo"])

                if 'profile' in resource_dict and 'ssignon' in resource_dict['profile']:
                    new_resource_panel.resource_info.ssignon_traits = resource.SSIGNONResourceTraits.from_dict(prefix="ssignon",source=resource_dict["profile"]["ssignon"])

                if 'profile' in resource_dict and 'cfdef' in resource_dict['profile']:
                    new_resource_panel.resource_info.cfdef_traits = resource.CfdefResourceTraits.from_dict(prefix="cfdef",source=resource_dict["profile"]["cfdef"])

                self.post_message(OpenTab(f"Resource: {search_query}",new_resource_panel))

                self.notify(f"Found resource profile: {search_query}")
            else:
                self.notify(f"Resource profile {search_query} couldn't be found")



commands = [
    #RACF commands
    "addgroup",
    "altgroup",
    "adduser", 
    "altuser",
    "addsd",
    "altdsd",
    "deluser",
    "delgroup",
    "deldsd",
    "connect",
    "remove",
    "permit",
    "racmap",
    "raclink",
    "racpriv",
    "racprmck",
    "racdcert",
    "rdefine",
    "ralter",
    "rdelete",
    "rlist",
    "listdsd",
    "listuser",
    "listgrp", 
    "display",
    "search",
    "setropts",
    "target",
    "rvary",

    "password",
    "passphrase",

    #TSO commands
    "PROFILE",
    "ALLOC",
    "EXEC",
    "DELETE",
    "STATUS",
    "LISTALC",
    "LISTBC",
    "LISTCAT",
    "LISTDS",
    "FREE",
    "SMCOPY",
    "OUTPUT",
    "PRINTDS",
    ]


from abc import ABC
from dataclasses import dataclass, fields
from typing import Any, Self


@dataclass
class TraitsBase(ABC):
    def to_traits(self, prefix: str) -> dict[str, Any]:
        traits = {}

        for field in fields(type(self)):
            value = getattr(self, field.name)
            field.metadata.get("masked", False)
            if value is not None:
                traits[f"{prefix}:{field.name}"] = value

        return traits

    @classmethod
    def from_dict(cls, prefix: str, source: dict[str, Any]) -> Self:
        kwargs = {}
        for field in fields(cls):
            key = f"{prefix}:{field.name}"
            value = source.get(key)
            if value is not None:
                kwargs[field.name] = value
        return cls(**kwargs)
    



import importlib.util
import json

from textual.app import App
from textual.containers import Container
from textual.signal import Signal
from textual.widgets import Footer, Header, Input, Label

from blackwall.messages import SubmitCommand
from blackwall.notifications import send_notification
from blackwall.settings import get_site_setting, get_user_setting
from blackwall.submit_command import execute_command

from .command_line import CommandLine
from .screens.modal.refresh import RefreshScreen
from .screens.modal.rvary import RvaryScreen
from .tabs import TabSystem
from .theme_3270 import ibm_3270_theme
from .theme_cynosure import cynosure_theme

zoau_enabled = importlib.util.find_spec('zoautil_py')

if zoau_enabled:
    from zoautil_py import zsystem  # type: ignore
else:
    print("##BLKWL_ERROR_1 Warning: could not find ZOAU, certain features will be disabled such as diplaying system and LPAR names")    

command_history = ""

#system information
if zoau_enabled:
    zsystem_info = json.loads(zsystem.zinfo()) # type: ignore
    system_name = zsystem_info["sys_info"]["sys_name"]
    lpar_name = zsystem_info["sys_info"]["lpar_name"]

class Blackwall(App):
    #Import css
    CSS_PATH = "UI.css"

    BINDINGS = [
        ("h", "push_screen('refresh')", "Switch to refresh screen"),
        ("r", "push_screen('rvary')", "Switch to rvary password screen"),
        ("ctrl+home", "go_to_cli", "Focus command line"),
    ]
    
    #This portion handles the text in the header bar
    def on_mount(self) -> None:
        self.title = "Blackwall Protocol"
        site_company = get_site_setting(section="meta",setting="company")
        if site_company is not None and site_company != "":
            self.sub_title = f"Mainframe Security Administration at {site_company}"
        else:
            self.sub_title = "Mainframe Security Administration"
        self.register_theme(cynosure_theme)
        #self.register_theme(ibm_3270_theme)
        user_theme = get_user_setting(section="display",setting="theme")
        if user_theme is not None or user_theme == "":
            try:
                self.theme = user_theme
            except ImportError:
                self.notify("Couldn't find user theme",severity="warning")
        else:
            self.theme = "cynosure"
        self.install_screen(RefreshScreen(), name="refresh")
        self.install_screen(RvaryScreen(), name="rvary")

        self.command_output_change = Signal(self,name="command_output_change")
        self.command_output = ""

        self.error_output_change = Signal(self,name="error_output_change")
        self.error_output = ""

    async def action_go_to_cli(self) -> None:
        """Focuses the command line"""
        cli = self.get_child_by_type(CommandLine).get_child_by_type(Input)
        cli.focus()

    async def on_submit_command(self, message: SubmitCommand) -> None:
        """Executes command from message"""
        if message.command != "":
            try:
                output = execute_command(message.command)
                if output is not None:
                    self.command_output = self.command_output + output
                    self.command_output_change.publish(data=self.command_output)
                    self.notify(f"command {message.command.upper()} successfully completed",severity="information")
            except BaseException as e:
                send_notification(self,message=f"Command {message.command.upper()} failed: {e}",severity="error")
                
    #UI elements
    def compose(self):
        #display system and LPAR name
        yield Header()
        if zoau_enabled:
            system_label = get_user_setting(section="display",setting="system_label")
            if system_label is not False:
                if get_user_setting(section="display",setting="short_system_label"):
                    yield Label(f"System: {system_name}, LPAR: {lpar_name}",classes="system-label")
                else:
                    yield Label(f"You are working on the {system_name} mainframe system in LPAR {lpar_name}",classes="system-label")
        yield CommandLine()
        with Container():
            yield TabSystem()
        yield Footer()




from textual.app import ComposeResult
from textual.containers import VerticalScroll
from textual.signal import Signal
from textual.widgets import Log


class PanelErrorHistory(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield Log(id="error_log")

    def on_mount(self) -> None:
        on_change: Signal[str] = self.app.error_output_change # type: ignore
        on_change.subscribe(node=self,callback=self.write_to_log)
        self.write_to_log(self.app.error_output) # type: ignore

    def write_to_log(self, output: str):
        log = self.get_child_by_id("#error_log",Log)
        log.clear()
        log.write(output)


from dataclasses import dataclass

from textual.app import ComposeResult
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll
from textual.reactive import reactive
from textual.widgets import Button, Input, Label

from blackwall.api import group
from blackwall.emoji import get_emoji
from blackwall.modals import generic_confirmation_modal
from blackwall.notifications import send_notification
from blackwall.panels.panel_mode import PanelMode
from blackwall.regex import racf_id_regex

from ..traits_ui import (
    generate_trait_section,
    get_traits_from_input,
    set_traits_in_input,
)


class PanelGroupInfo(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Creation date:",classes="date-labels")
        yield Input(id="base_create_date",disabled=True,classes="date-fields")

class PanelGroupNameAndSubgroup(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Group name:")
        yield Input(id="group_name",restrict=racf_id_regex,max_length=8,classes="field-short-generic",tooltip="1-8 character long alphanumeric name used to identify the group")
        yield Label("Superior group:")
        yield Input(max_length=8,restrict=racf_id_regex,id="base_superior_group",classes="field-short-generic",tooltip="")
        yield Label("Owner:")
        yield Input(max_length=8,restrict=racf_id_regex,id="base_owner",classes="field-short-generic",tooltip="")

class PanelGroupInstallationData(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Installation data:")
        yield Input(max_length=255,id="base_installation_data",classes="installation-data",tooltip="Optional used defined data. This can be used to put in a description about what the group is used for. Can't be more than 255 characters long.")

class PanelGroupDatasetModel(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Dataset model:")
        yield Input(id="base_data_set_model",classes="field-long-generic")

class PanelGroupSegments(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield from generate_trait_section(title="DFP segment", prefix="dfp", traits_class=group.DFPGroupTraits)

class PanelGroupActionButtons(HorizontalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    delete_is_disabled = edit_mode is not True

    def __init__(self, save_action: str, delete_action: str):
        super().__init__()
        self.save_action = save_action
        self.delete_action = delete_action
    
    def compose(self) -> ComposeResult:
        yield Button("Create",id="save",action="save",classes="action-button")
        yield Button("Delete",id="delete",action="delete",variant="error",classes="action-button",disabled=self.delete_is_disabled)

    async def action_save(self):
        await self.app.run_action(self.save_action,default_namespace=self.parent)

    async def action_delete(self):
        await self.app.run_action(self.delete_action,default_namespace=self.parent)

@dataclass
class GroupInfo:
    base_traits: group.BaseGroupTraits | None = None
    dfp_traits: group.DFPGroupTraits | None = None

    group_name: str = ""

class PanelGroup(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelGroupInfo()
        yield PanelGroupNameAndSubgroup()
        yield PanelGroupInstallationData()
        yield PanelGroupDatasetModel()
        yield PanelGroupSegments()
        yield PanelGroupActionButtons(save_action="save_group",delete_action="delete_group")

    group_info: reactive[GroupInfo] = reactive(GroupInfo())

    def set_edit_mode(self):
        self.query_exactly_one("#group_name",Input).disabled = True
        self.query_exactly_one("#delete",Button).disabled = False
        self.query_exactly_one("#save",Button).label = f"{get_emoji("💾")} Save"
        self.notify("Switched to edit mode",severity="information")

    def on_mount(self) -> None:
        if group.group_exists(self.group_info.group_name):
            self.query_exactly_one("#group_name",Input).value = self.group_info.group_name
            if self.group_info.base_traits is not None:
                set_traits_in_input(self,traits=self.group_info.base_traits,prefix="base")
            
            if self.group_info.dfp_traits is not None:
                set_traits_in_input(self,traits=self.group_info.dfp_traits,prefix="dfp")
            self.set_edit_mode()

    def action_delete_group_api(self) -> None:
        group_name = self.get_child_by_type(PanelGroupNameAndSubgroup).get_child_by_id("group_name",Input).value
        if group.group_exists(group_name):
            message, return_code = group.delete_group(group=group_name)
            
            if (return_code == 0):
                self.notify(f"Group {group_name} deleted, return code: {return_code}",severity="warning")
            else:
                self.notify(f"{message}, return code: {return_code}",severity="error")

    def action_delete_group(self) -> None:
        group_name = self.get_child_by_type(PanelGroupNameAndSubgroup).get_child_by_id("group_name",Input).value
        generic_confirmation_modal(self,modal_text=f"Are you sure you want to delete group {group_name}?",confirm_action="delete_group_api",action_widget=self)

    def action_save_group(self) -> None:
        group_name = self.get_child_by_type(PanelGroupNameAndSubgroup).get_child_by_id("group_name",Input).value
        group_exists = group.group_exists(group=group_name)

        operator = "alter" if group_exists else "add"

        base_segment = get_traits_from_input(operator, self, prefix="base", trait_cls=group.BaseGroupTraits)
        dfp_segment = get_traits_from_input(operator, self, prefix="dfp", trait_cls=group.DFPGroupTraits)
        result = group.update_group(
            group=group_name,
            create=not group_exists,
            base=base_segment,
            dfp=dfp_segment,
        )

        if not group_exists:
            if (result == 0 or result == 4):
                self.notify(f"Group {group_name} created, return code: {result}",severity="information")
            else:
                send_notification(self,message=f"Unable to create group, return code: {result}",severity="error")
        else:
            if (result == 0):
                self.notify(f"Group {group_name} updated, return code: {result}",severity="information")
            else:
                send_notification(self,message=f"Unable to update group, return code: {result}",severity="error")


from textual import on
from textual.app import ComposeResult
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll
from textual.widgets import Button, DataTable, Input, Label, Select

from blackwall.api import group, permit, resource
from blackwall.emoji import get_emoji
from blackwall.notifications import send_notification
from blackwall.panels.traits_ui import get_traits_from_input
from blackwall.regex import racf_id_regex

PERMIT_COLUMNS = [
    ("ID", "Type", "Access"),
]

class PanelResourcePermitInfo(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Use this panel to create, delete, and update permits for general resource profiles",classes="label-generic")

class PanelResourcePermitSearchField(HorizontalGroup):
    def __init__(self, search_action: str):
        super().__init__()
        self.search_action = search_action

    def compose(self) -> ComposeResult:
        yield Input(id="search_permit_class",placeholder="class...",classes="field-short-generic")
        yield Input(id="search_permit_profile",placeholder="profile name...",classes="search-field")    
        yield Button(label="Get ACL",id="search_permit_button",action="search")

    @on(Input.Submitted)
    async def action_search(self):
        await self.app.run_action(self.search_action,default_namespace=self.parent)

class PanelResourcePermitCreate(HorizontalGroup):
    def __init__(self, update_action: str):
        super().__init__()
        self.update_action = update_action
    
    def compose(self) -> ComposeResult:
        yield Select([("NONE", "NONE"),("READ", "READ"),("EXECUTE", "EXECUTE"),("UPDATE", "UPDATE"),("CONTROL", "CONTROL"),("ALTER", "ALTER")],value="READ",classes="uacc-select",id="base_access")
        yield Input(id="permit_racf_id",placeholder="ID...",max_length=8,restrict=racf_id_regex,classes="field-short-generic", tooltip="User ID or group ID you want this permit change to affect")    
        yield Button(f"{get_emoji("💾")} Save",id="resource_permit_save",action="update")

    @on(Input.Submitted)
    async def action_create(self):
        await self.app.run_action(self.update_action,default_namespace=self.parent)

class PanelResourcePermitList(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Current permits:",classes="label-generic")
        yield DataTable(id="resource_permits_table")

    def on_mount(self) -> None:
        permit_table = self.get_child_by_id("resource_permits_table",DataTable)
        permit_table.zebra_stripes = True
        permit_table.add_columns(*PERMIT_COLUMNS[0]) 

class PanelResourcePermit(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelResourcePermitInfo()
        yield PanelResourcePermitSearchField(search_action="search")
        yield PanelResourcePermitCreate(update_action="update")
        yield PanelResourcePermitList()

    def get_acl(self, notification: bool) -> None:
        search_profile_field_value = self.get_child_by_type(PanelResourcePermitSearchField).get_child_by_id("search_permit_profile",Input).value
        search_class_field_value = self.get_child_by_type(PanelResourcePermitSearchField).get_child_by_id("search_permit_class",Input).value
        permit_table = self.get_child_by_type(PanelResourcePermitList).get_child_by_id("resource_permits_table",DataTable)
        
        if resource.resource_profile_exists(resource=search_profile_field_value,resource_class=search_class_field_value):
            resource_acl = resource.get_resource_acl(resource=search_profile_field_value,resource_class=search_class_field_value)
            permit_table.clear(columns=False)

            for entry in resource_acl:
                entry_id = entry["base:access_id"]
                entry_access = entry["base:access_type"]

                #Checks if the entry is a user or group
                id_type = "group" if group.group_exists(entry_id) else "user"
                
                #Adds the entry to the datatable
                permit_table.add_row(entry_id,id_type,entry_access)
            if notification:
                self.notify(f"Found profile {search_profile_field_value} in class {search_class_field_value}",severity="information")
        else:
            if notification:
                self.notify(f"Couldn't find profile {search_profile_field_value} in class {search_class_field_value}",severity="error")

    def action_search(self) -> None:
        self.get_acl(notification=True)

    def action_update(self) -> None:
        search_profile_field_value = self.get_child_by_type(PanelResourcePermitSearchField).get_child_by_id("search_permit_profile",Input).value
        search_class_field_value = self.get_child_by_type(PanelResourcePermitSearchField).get_child_by_id("search_permit_class",Input).value

        racf_id_field_value = self.get_child_by_type(PanelResourcePermitCreate).get_child_by_id("permit_racf_id",Input).value

        if resource.resource_profile_exists(resource=search_profile_field_value,resource_class=search_class_field_value):
            base_segment = get_traits_from_input("alter", self, prefix="base", trait_cls=permit.BasePermitTraits)

            return_code = permit.update_resource_permit(profile=search_profile_field_value,class_name=search_class_field_value,racf_id=racf_id_field_value,base=base_segment)

            self.get_acl(notification=False)

            if return_code == 0:
                self.notify("Created permit",severity="information")
            else:
                send_notification(self,message=f"Couldn't create permit, return code: {return_code}",severity="error")
                



from enum import Enum

from blackwall.api.dataset import dataset_profile_exists, get_dataset_profile
from blackwall.api.group import get_group, group_exists
from blackwall.api.resource import get_resource_profile, resource_profile_exists
from blackwall.api.setropts import get_active_classes
from blackwall.api.user import get_user, user_exists


class QueryType(Enum):
    User = "user"
    Group = "group"
    Dataset = "dataset"
    Resource = "resource"

    @classmethod
    def all(cls) -> set["QueryType"]:
        return {QueryType.User,QueryType.Group,QueryType.Dataset,QueryType.Resource}

def is_id(query: str):
    return len(query) <= 8 and query.isalnum()

def search_user(query: str) -> dict | None:
    if is_id(query) and user_exists(query):
        return get_user(query)
        
def search_group(query: str) -> dict | None:
    if is_id(query) and group_exists(query):
        return get_group(query)

def search_dataset(query: str) -> dict | None:
    if dataset_profile_exists(query):
        return get_dataset_profile(query)

def search_resource(query: str, class_name: str | None) -> dict | None:
    if class_name is str:
        if resource_profile_exists(resource=query,resource_class=class_name):
            return get_resource_profile(resource=query,resource_class=class_name)
    else:
        active_classes = get_active_classes()
        for r_class in active_classes:
            if resource_profile_exists(resource=query,resource_class=r_class):
                return get_resource_profile(resource=query,resource_class=r_class)

def search_database_query_one(query: str, query_types: set[QueryType], class_name: str | None = None):
    results = {}
    for query_type in query_types:
        if query_type is QueryType.User:
            results[query_type] = search_user(query)
        elif query_type is QueryType.Group:
            results[query_type] = search_group(query)
        elif query_type is QueryType.Dataset:
            results[query_type] = search_dataset(query)
        elif query_type is QueryType.Resource:
            results[query_type] = search_resource(query,class_name)
    
    return results

def search_database_query_multiple(query: str, query_types: set[QueryType], class_name: str | None = None):
    pass




import random

from blackwall.settings import get_user_setting

emoji_allowed = get_user_setting(section="display",setting="emojis")

def get_emoji(emoji: str | list[str]) -> str:
    if emoji_allowed is not False:
        if emoji is type(str):
            return emoji
        else:
            return random.choice(emoji)
    else:
        return ""


#General resource API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

import importlib.util
from dataclasses import dataclass, field

from .traits_base import TraitsBase

#Checks if RACFU can be imported
racfu_enabled = importlib.util.find_spec('racfu')

if racfu_enabled:
    from racfu import racfu  # type: ignore
else:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")       

@dataclass
class BaseResourceTraits(TraitsBase):
    #add+alter fields
    owner: str | None = field(default=None,metadata={"label": "Owner", "allowed_in": {"add","alter"}})
    audit_alter: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_control: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_none: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_read: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_read: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_update: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    security_category: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    installation_data: str | None = field(default=None,metadata={"label": "Installation data", "allowed_in": {"add","alter"}})
    level: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    member_class_name: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    notify_userid: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    security_label: str | None = field(default=None,metadata={"label": "Security label", "allowed_in": {"add","alter"}})
    security_level: str | None = field(default=None,metadata={"label": "Security level", "allowed_in": {"add","alter"}})
    single_data_set_tape_volume: bool | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    time_zone: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    universal_access: str | None = field(default=None,metadata={"label": "UACC", "allowed_in": {"add","alter"}})
    warn_on_insufficient_access: bool | None = field(default=None,metadata={"label": "Warn on insufficient access", "allowed_in": {"add","alter"}})
    terminal_access_allowed_day: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    terminal_access_allowed_time: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})

    tape_vtoc: bool | None = field(default=None,metadata={"label": "Tape VTOC", "allowed_in": {"add","alter","extract"}})

    #add fields
    model_profile: str | None = field(default=None,metadata={"allowed_in": {"add"}})
    model_profile_class: str | None = field(default=None,metadata={"allowed_in": {"add"}})
    model_profile_generic: bool | None = field(default=None,metadata={"allowed_in": {"add"}})
    model_profile_volume: str | None = field(default=None,metadata={"allowed_in": {"add"}})

    #alter fields
    global_audit_alter: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_control: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_none: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_read: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_update: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    volume: str | None = field(default=None,metadata={"allowed_in": {"alter"}})

    #extraction fields
    access_list: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    access_count: int | None = field(default=None,metadata={"allowed_in": {"extract"}})
    access_type: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    access_id: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    alter_access_count: int | None = field(default=None,metadata={"allowed_in": {"extract"}})
    control_access_count: int | None = field(default=None,metadata={"allowed_in": {"extract"}})
    read_access_count: int | None = field(default=None,metadata={"allowed_in": {"extract"}})
    update_access_count: int | None = field(default=None,metadata={"allowed_in": {"extract"}})
    security_categories: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    create_date: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    is_generic: bool | None = field(default=None,metadata={"allowed_in": {"extract"}})
    last_change_date: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    last_reference_date: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    member_class_names: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    auditing: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    global_auditing: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    volumes: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    terminal_access_allowed_days: str | None = field(default=None,metadata={"allowed_in": {"extract"}})

@dataclass
class KerbResourceTraits(TraitsBase):
    validate_addresses: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    default_ticket_life: int | None = field(default=None,metadata={"label": "Default ticket life","allowed_in": {"add","alter","extract"}})
    max_ticket_life: int | None = field(default=None,metadata={"label": "Max ticket life","allowed_in": {"add","alter","extract"}})
    encryption_algorithm: str | None = field(default=None,metadata={"label": "Encryption algorithm","allowed_in": {"add","alter","extract"}})
    encryption_algorithms: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    realm_name: str | None = field(default=None,metadata={"label": "Realm name","allowed_in": {"add","alter","extract"}})
    key_version: str | None = field(default=None,metadata={"label": "Key version","allowed_in": {"extract"}})
    min_ticket_life: list[str] | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    password: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})

@dataclass
class DLFDataResourceTraits(TraitsBase):
    job_name: str | None = field(default=None,metadata={"label": "Job name","allowed_in": {"add","alter","extract"}})
    job_names: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    retain_object_after_use: bool | None = field(default=None,metadata={"label": "Retain object after use","allowed_in": {"add","alter"}})

@dataclass
class EIMResourceTraits(TraitsBase):
    domain_distinguished_name: str | None = field(default=None,metadata={"label": "Domain distinguished name","allowed_in": {"add","alter","extract"}})
    kerberos_registry: str | None = field(default=None,metadata={"label": "Kerberos registry","allowed_in": {"add","alter","extract"}})
    local_registry: str | None = field(default=None,metadata={"label": "Local registry","allowed_in": {"add","alter","extract"}})
    options: str | None = field(default=None,metadata={"label": "Options","allowed_in": {"add","alter","extract"}})

@dataclass
class JESResourceTraits(TraitsBase):
    icsf_key_label: str | None = field(default=None,metadata={"label": "ICSF key label","allowed_in": {"add","alter","extract"}})

@dataclass
class ICSFResourceTraits(TraitsBase):
    certificate_label: str | None = field(default=None,metadata={"label": "Certificate label","allowed_in": {"add","alter","extract"}})
    certificate_labels: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    exportable_public_keys: str | None = field(default=None,metadata={"label": "Exportable public keys","allowed_in": {"add","alter","extract"}})
    symmetric_export_public_key: str | None = field(default=None,metadata={"label": "Symmetric export public key","allowed_in": {"add","alter","extract"}})
    symmetric_export_public_keys: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    symmetric_cpacf_rewrap: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    symmetric_cpacf_rewrap_return: bool | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    asymetric_key_usage: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    key_usage_options: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})

@dataclass
class ICTXResourceTraits(TraitsBase):
    identity_map_timeout: int | None = field(default=None,metadata={"label": "Identity map timeout","allowed_in": {"add","alter","extract"}})
    use_identity_map: bool | None = field(default=None,metadata={"label": "Use identity map","allowed_in": {"add","alter","extract"}})
    require_identity_mapping: bool | None = field(default=None,metadata={"label": "Require identity mapping","allowed_in": {"add","alter","extract"}})
    cache_application_provided_identity_map: bool | None = field(default=None,metadata={"label": "Cache application provided identity map","allowed_in": {"add","alter","extract"}})

@dataclass
class IDTPARMSResourceTraits(TraitsBase):
    signature_algorithm: str | None = field(default=None,metadata={"label": "Signature algorithm", "allowed_in": {"add","alter","extract"}})
    identity_token_timeout: int | None = field(default=None,metadata={"label": "Identity token timeout","allowed_in": {"add","alter","extract"}})
    use_for_any_application: str | None = field(default=None,metadata={"label": "Use for any application","allowed_in": {"add","alter","extract"}})

@dataclass
class SessionResourceTraits(TraitsBase):
    security_checking_level: str | None = field(default=None,metadata={"label": "Security checking level","allowed_in": {"add","alter","extract"}})
    session_key_interval: int | None = field(default=None,metadata={"label": "Session key interval","allowed_in": {"add","alter","extract"}})
    session_key: str | None = field(default=None,metadata={"label": "Session key","allowed_in": {"add","alter","extract"}})
    locked: bool | None = field(default=None,metadata={"label": "Locked","allowed_in": {"add","alter","extract"}})

@dataclass
class SVFMRResourceTraits(TraitsBase):
    parameter_list_name: str | None = field(default=None,metadata={"label": "Parameter list name","allowed_in": {"add","alter","extract"}})
    script_name: str | None = field(default=None,metadata={"label": "Script name","allowed_in": {"add","alter","extract"}})

@dataclass
class STDATAResourceTraits(TraitsBase):
    userid: str | None = field(default=None,metadata={"label": "User ID","allowed_in": {"add","alter","extract"}})
    group: str | None = field(default=None,metadata={"label": "Group","allowed_in": {"add","alter","extract"}})
    privileged: bool | None = field(default=None,metadata={"label": "Privileged","allowed_in": {"add","alter","extract"}})
    trace: bool | None = field(default=None,metadata={"label": "Trace","allowed_in": {"add","alter","extract"}})
    trusted: bool | None = field(default=None,metadata={"label": "Trusted","allowed_in": {"add","alter","extract"}})

@dataclass
class ProxyResourceTraits(TraitsBase):
    bind_distinguished_name: str | None = field(default=None,metadata={"label": "Bind distinguished name","allowed_in": {"add","alter","extract"}})
    bind_password: str | None = field(default=None,metadata={"label": "Bind password","allowed_in": {"add","alter"},"input_args": {"password": True}})
    ldap_host: str | None = field(default=None,metadata={"label": "LDAP host","allowed_in": {"add","alter","extract"}})

@dataclass
class MFPolicyResourceTraits(TraitsBase):
    factor: str | None = field(default=None,metadata={"label": "Factor","allowed_in": {"add","alter","extract"}})
    factors: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    token_timeout: int | None = field(default=None,metadata={"label": "Token timeout","allowed_in": {"extract"}})
    reuse_token: bool | None = field(default=None,metadata={"label": "Re-use token","allowed_in": {"add","alter","extract"}})

@dataclass
class SIGVERResourceTraits(TraitsBase):
    fail_program_load_condition: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    log_signature_verification_events: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    signature_required: bool | None = field(default=None,metadata={"label": "Signature required","allowed_in": {"add","alter","extract"}})

@dataclass
class CDTINFOResourceTraits(TraitsBase):
    case_allowed: str | None = field(default=None,metadata={"label": "Case allowed", "allowed_in": {"add","alter","extract"}})
    default_racroute_return_code: str | None = field(default=None,metadata={"label": "Default racroute return code", "allowed_in": {"add","alter","extract"}})
    valid_first_character: str | None = field(default=None,metadata={"label": "Valid first character","allowed_in": {"add","alter","extract"}})
    valid_first_characters: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    generic_profile_checking: str | None = field(default=None,metadata={"label": "Generic profile checking","allowed_in": {"add","alter","extract"}})
    generic_profile_sharing: str | None = field(default=None,metadata={"label": "Generic profile sharing","allowed_in": {"add","alter","extract"}})
    grouping_class_name: str | None = field(default=None,metadata={"label": "Grouping class name","allowed_in": {"add","alter","extract"}})
    key_qualifiers: str | None = field(default=None,metadata={"label": "Key qualifiers","allowed_in": {"add","alter","extract"}})
    manditory_access_control_processing: str | None = field(default=None,metadata={"label": "Mandatory access control processing","allowed_in": {"add","alter","extract"}})
    max_length: int | None = field(default=None,metadata={"label": "Max length", "allowed_in": {"add","alter","extract"}})
    max_length_entityx: int | None = field(default=None,metadata={"label": "Max length entityx", "allowed_in": {"add","alter","extract"}})
    member_class_name: str | None = field(default=None,metadata={"label": "Member class name","allowed_in": {"add","alter","extract"}})
    operations: str | None = field(default=None,metadata={"label": "Operations","allowed_in": {"add","alter","extract"}})
    valid_other_character: str | None = field(default=None,metadata={"label": "Valid other character","allowed_in": {"add","alter","extract"}})
    valid_other_characters: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    posit_number: int | None = field(default=None,metadata={"label": "Posit number","allowed_in": {"add","alter","extract"}})
    profiles_allowed: str | None = field(default=None,metadata={"label": "Profiles allowed","allowed_in": {"add","alter","extract"}})
    raclist_allowed: str | None = field(default=None,metadata={"label": "Raclist allowed","allowed_in": {"add","alter","extract"}})
    send_enf_signal_on_profile_creation: str | None = field(default=None,metadata={"label": "Send enf signal on profile creation", "allowed_in": {"add","alter","extract"}})
    security_label_required: str | None = field(default=None,metadata={"label": "Security label required","allowed_in": {"add","alter","extract"}})
    default_universal_access: str | None = field(default=None,metadata={"label": "Default universal access","allowed_in": {"add","alter","extract"}})

@dataclass
class TMEResourceTraits(TraitsBase):
    child: str | None = field(default=None,metadata={"label": "Child","allowed_in": {"add","alter","extract"}})
    group: str | None = field(default=None,metadata={"label": "Group","allowed_in": {"add","alter","extract"}})
    parent: str | None = field(default=None,metadata={"label": "Parent","allowed_in": {"add","alter","extract"}})
    resource: str | None = field(default=None,metadata={"label": "Resource","allowed_in": {"add","alter","extract"}})
    role: str | None = field(default=None,metadata={"label": "Role","allowed_in": {"add","alter","extract"}})

    children: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    groups: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    resources: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    roles: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})

@dataclass
class SSIGNONResourceTraits(TraitsBase):
    enhanced_pass_ticket_label: str | None = field(default=None,metadata={"label": "Enchaned pass ticket label","allowed_in": {"add","alter","extract"}})
    enhanced_pass_ticket_type: str | None = field(default=None,metadata={"label": "Enchaned pass ticket type","allowed_in": {"add","alter","extract"}})
    enhanced_pass_ticket_timeout: str | None = field(default=None,metadata={"label": "Enchaned pass ticket timeout","allowed_in": {"add","alter","extract"}})
    enhanced_pass_ticket_replay: str | None = field(default=None,metadata={"label": "Enchanced pass ticket replay","allowed_in": {"add","alter","extract"}})
    legacy_pass_ticket_label: str | None = field(default=None,metadata={"label": "Legacy pass ticket label","allowed_in": {"add","alter","extract"}})

    mask_legacy_pass_ticket_key: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    encrypt_legacy_pass_ticket_key: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})

@dataclass
class CfdefResourceTraits(TraitsBase):
    custom_field_data_type: str | None = field(default=None,metadata={"label": "Custom field data type","allowed_in": {"add","extract"}})
    valid_first_characters: str | None = field(default=None,metadata={"label": "Valid first characters","allowed_in": {"add","alter","extract"}})
    help_text: str | None = field(default=None,metadata={"label": "Help text","allowed_in": {"add","alter","extract"}})
    list_heading_text: str | None = field(default=None,metadata={"label": "List heading text","allowed_in": {"add","alter","extract"}})
    mixed_case_allowed: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    min_numeric_value: int | None = field(default=None,metadata={"label": "Min numeric value","allowed_in": {"add","alter","extract"}})
    max_field_length: int | None = field(default=None,metadata={"label": "Max field length","allowed_in": {"add","alter","extract"}})
    max_numeric_value: int | None = field(default=None,metadata={"label": "Max numeric value","allowed_in": {"add","alter","extract"}})
    valid_other_characters: str | None = field(default=None,metadata={"label": "Valid other characters","allowed_in": {"add","alter","extract"}})
    validation_rexx_exec: str | None = field(default=None,metadata={"label": "Validation rexx exec","allowed_in": {"add","alter","extract"}})

#General resource profile function
def resource_profile_exists(resource_class: str,resource: str) -> bool:
    """Checks if a general resource profile exists, returns true or false"""
    if racfu_enabled:
        """Checks if a general resource profile exists, returns true or false"""
        result = racfu({"operation": "extract", "admin_type": "resource", "profile_name": resource.upper(), "class_name": resource_class.upper()}) # type: ignore
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return False

def get_resource_profile(resource_class: str, resource: str) -> dict:
    """Returns a dict with information about the resource profile"""
    if racfu_enabled:
        """Doesn't handle general resource profiles that don't exist, recommend using resource_profile_exists() first"""
        result = racfu({"operation": "extract", "admin_type": "resource", "profile_name": resource.upper(), "class_name": resource_class.upper()}) # type: ignore
        return result.result
    else:
        return {}
    
def get_resource_acl(resource_class: str, resource: str) -> list[dict]:
    """Returns a string list with the access list of the specified resource"""
    if racfu_enabled:
        """Returns a list of active classes on the system"""
        result = racfu({"operation": "extract", "admin_type": "resource", "profile_name": resource.upper(), "class_name": resource_class.upper()}) # type: ignore
        return result.result["profile"]["base"]["base:access_list"] # type: ignore
    else:
        return []

def update_resource_profile(
        resource_class: str,
        resource: str, 
        create: bool, 
        base: BaseResourceTraits,
        stdata: STDATAResourceTraits | None = None,
        cdtinfo: CDTINFOResourceTraits | None = None,
        kerb: KerbResourceTraits | None = None,
        tme: TMEResourceTraits | None = None,
        ssignon: SSIGNONResourceTraits | None = None,
        proxy: ProxyResourceTraits | None = None,
        icsf: ICSFResourceTraits | None = None,
        ictx: ICTXResourceTraits | None = None,
        svfmr: SVFMRResourceTraits | None = None,
        sigver: SIGVERResourceTraits | None = None,
        mfpolicy: MFPolicyResourceTraits | None = None,
        session: SessionResourceTraits | None = None,
        idtparms: IDTPARMSResourceTraits | None = None,
        jes: JESResourceTraits | None = None,
        dlfdata: DLFDataResourceTraits | None = None,
        cfdef: CfdefResourceTraits | None = None,
        eim: EIMResourceTraits | None = None,
        ):
    traits = base.to_traits(prefix="base")

    if stdata is not None:
        traits.update(stdata.to_traits("stdata"))

    if cdtinfo is not None:
        traits.update(cdtinfo.to_traits("cdtinfo"))

    if ssignon is not None:
        traits.update(ssignon.to_traits("ssignon"))

    if sigver is not None:
        traits.update(sigver.to_traits("sigver"))

    if svfmr is not None:
        traits.update(svfmr.to_traits("svfmr"))

    if tme is not None:
        traits.update(tme.to_traits("tme"))

    if kerb is not None:
        traits.update(kerb.to_traits("kerb"))

    if proxy is not None:
        traits.update(proxy.to_traits("proxy"))

    if icsf is not None:
        traits.update(icsf.to_traits("icsf"))

    if ictx is not None:
        traits.update(ictx.to_traits("ictx"))

    if mfpolicy is not None:
        traits.update(mfpolicy.to_traits("mfpolicy"))

    if session is not None:
        traits.update(session.to_traits("session"))

    if idtparms is not None:
        traits.update(idtparms.to_traits("idtparms"))

    if jes is not None:
        traits.update(jes.to_traits("jes"))

    if dlfdata is not None:
        traits.update(dlfdata.to_traits("dlfdata"))

    if cfdef is not None:
        traits.update(cfdef.to_traits("cfdef"))

    if eim is not None:
        traits.update(eim.to_traits("eim"))

    operation = "add" if create else "alter"
    
    result = racfu( # type: ignore
        {
            "operation": operation, 
            "admin_type": "resource", 
            "profile_name": resource.upper(),
            "class_name": resource_class.upper(),
            "traits":  traits,
        },
    )
    return result.result["return_codes"]["racf_return_code"]

def delete_resource_profile(resource_class: str,resource: str) -> tuple[str, int]:
    if racfu_enabled:
        result = racfu( # type: ignore
                {
                    "operation": "delete", 
                    "admin_type": "resource", 
                    "profile_name": resource.upper(),
                    "class_name": resource_class.upper(),
                },
            )
        #TODO add error message
        return "", result.result["return_codes"]["racf_return_code"]
    else:
        return "RACFu can't be found", 8



from textual.message import Message
from textual.widget import Widget


class OpenTab(Message):
    def __init__(self, title: str, content: Widget):
        super().__init__()
        self.title =  title
        self.content = content

class SubmitCommand(Message):
    def __init__(self, command: str):
        super().__init__()
        self.command =  command

class SubmitError(Message):
    def __init__(self, error: str):
        super().__init__()
        self.error = error



from textual import on
from textual.app import ComposeResult
from textual.containers import HorizontalGroup
from textual.suggester import SuggestFromList
from textual.widgets import Input

from blackwall.commands_definition import commands
from blackwall.messages import SubmitCommand
from blackwall.settings import get_user_setting


class CommandLine(HorizontalGroup):
    BINDINGS = [
        ("up", "cycle_up", "Cycle previous executed commands"),
        ("down", "cycle_down", "Cycle previous executed commands"),
    ]

    command_history_list = []
    current_command_history_entry = -1

    def compose(self) -> ComposeResult:
        yield Input(id="cli",max_length=250,placeholder="Submit a TSO/RACF command...",classes="commands",suggester=SuggestFromList(commands,case_sensitive=False),tooltip="Use this command field to submit TSO and RACF commands. You can view the output in the command history panel")

    @on(Input.Submitted)
    def submit_command(self) -> None:
        command_line = self.get_child_by_id("cli",Input)
        command = command_line.value
        self.post_message(SubmitCommand(command))
        clear_on_submission = get_user_setting(section="commands",setting="clear_on_submission")
        if clear_on_submission is not False:
            self.command_history_list.append(command)
            command_line.value = ""

    def action_cycle_up(self) -> None:
        if len(self.command_history_list) > 0:
            command_line = self.get_child_by_id("cli",Input)
            command_line.value = self.command_history_list[self.current_command_history_entry]
            if self.current_command_history_entry < len(self.command_history_list) -1:
                self.current_command_history_entry = self.current_command_history_entry + 1

    def action_cycle_down(self) -> None:
        if len(self.command_history_list) > 0:
            command_line = self.get_child_by_id("cli",Input)
            command_line.value = self.command_history_list[self.current_command_history_entry]
            if self.current_command_history_entry > -1:
                self.current_command_history_entry = self.current_command_history_entry - 1



import importlib.util
from dataclasses import dataclass, field
from typing import Any

from .traits_base import TraitsBase

#Checks if RACFU can be imported
racfu_enabled = importlib.util.find_spec('racfu')

if racfu_enabled:
    from racfu import racfu  # type: ignore
else:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")       

@dataclass
class BaseSetroptsTraits(TraitsBase):

    application_logon_auditing: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    uncataloged_data_set_access: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    log_racf_command_violations: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    enhanced_generic_naming: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    erase_data_sets_on_delete_all: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    erase_data_sets_on_delete_security_level: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    generic_profile_sharing_classes: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    generic_owner: bool | None = field(default=None,metadata={"allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    list_of_groups_access_checking: bool | None = field(default=None,metadata={"allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    password_history: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    revoke_inactive_userids_interval: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    max_password_change_interval: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    #jes_network_user: str | None = field(default=None,metadata={"label": "JES network user", "allowed_in": {"alter","extract"},"input_args": {"classes": "field-short-generic","max_length": 8,}})
    jes_undefined_user: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    kerberos_encryption_level: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    audit_log_always_classes: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    audit_log_default_classes: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    audit_log_failure_classes: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    audit_log_never_classes: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    audit_log_success_classes: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    min_password_change_interval: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    multi_level_security_address_space: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    multi_level_security_file_system: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    multi_level_security_interprocess: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    multi_level_security_declassification: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    passphrase_change_interval: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    data_set_single_level_name_prefix_protection: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    primary_language: str | None = field(default=None,metadata={"label": "Primary language", "allowed_in": {"alter","extract"},"input_args": {"classes": "field-short-generic","max_length": 8}})
    secondary_language: str | None = field(default=None,metadata={"label": "Secondary language", "allowed_in": {"alter","extract"},"input_args": {"classes": "field-short-generic","max_length": 8}})
    protect_all_data_sets: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    password_encryption_algorithm: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    refresh: bool | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    tape_data_set_security_retention_period: int | None = field(default=None,metadata={"label": "Tape dataset security retention period","allowed_in": {"alter","extract"},"input_args": {"classes": "field-medium-generic"}})
    max_incorrect_password_attempts: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    rvary_status_password_format: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    rvary_status_password: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    rvary_status_password_format: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    #log_commands_issuesd_by_special_users: bool | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    max_session_key_interval: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})

    #security_level_auditing: str | None = field(default=None,metadata={"label": "Security level auditing", "allowed_in": {"alter","extract"}})
    
    terminal_universal_access: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    password_expiration_warning: str | None = field(default=None,metadata={"allowed_in": {"alter","extract"}})
    

    #Bools
    #add_creator_to_access_list: bool | None = field(default=None,metadata={"label": "Add creator to access list", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    #Note type in documentation wrong
    #automatic_data_set_protection: bool | None = field(default=None,metadata={"label": "Automatic dataset protection", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    multi_level_security_file_names: bool | None = field(default=None,metadata={"label": "Multi-level security file names", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    multi_level_security_logon: bool | None = field(default=None,metadata={"label": "Multi-level security logon", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    security_label_compatibility_mode: bool | None = field(default=None,metadata={"label": "Security label compatibility mode", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    program_control: bool | None = field(default=None,metadata={"label": "Program control", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    tape_data_set_protection: bool | None = field(default=None,metadata={"label": "Tape dataset protection","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    #erase_data_sets_on_delete: bool | None = field(default=None,metadata={"label": "Erase datasets on deletion","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    log_real_data_set_name: bool | None = field(default=None,metadata={"label": "Log real dataset names", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    log_operator_actions: bool | None = field(default=None,metadata={"label": "Log operator actions","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    security_label_auditing: bool | None = field(default=None,metadata={"label": "Security label auditing", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    security_label_system: bool | None = field(default=None,metadata={"label": "Security label system", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    security_label_control: bool | None = field(default=None,metadata={"label": "Security label control", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    #record_user_verification_statistics: bool | None = field(default=None,metadata={"label": "Record user verification statistics", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    password_rules: bool | None = field(default=None,metadata={"label": "Password rules", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    mixed_case_password_support: bool | None = field(default=None,metadata={"label": "Mixed case password support", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    #special_character_password_support: bool | None = field(default=None,metadata={"label": "Special character password support", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    multi_level_security_label_alteration: bool | None = field(default=None,metadata={"label": "Multi-level security label alteration","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    profile_modelling: bool | None = field(default=None,metadata={"label": "Profile modelling","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    profile_modelling_generation_data_group: bool | None = field(default=None,metadata={"label": "Profile modelling generation data group","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    profile_modelling_group: bool | None = field(default=None,metadata={"label": "Profile modelling group","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    profile_modelling_user: bool | None = field(default=None,metadata={"label": "Profile modelling user","allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    jes_batch: bool | None = field(default=None,metadata={"label": "JES batch", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    jes_early_verification: bool | None = field(default=None,metadata={"label": "JES early verification", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})
    jes_execution_batch_monitoring: bool | None = field(default=None,metadata={"label": "JES execution monitoring", "allowed_in": {"alter","extract"}, "input_args": {"classes": "generic-checkbox-medium"}})

    active_classes: list[str] | None = field(default=None,metadata={"label": "Active classes", "input_args": {"classes": "list-view-generic"}, "allowed_in": {"alter","extract"}})
    statistics_classes: list[str] | None = field(default=None,metadata={"label": "Statistics classes", "allowed_in": {"alter","extract"}})
    generic_command_classes: list[str] | None = field(default=None,metadata={"label": "Generic command classes", "allowed_in": {"alter","extract"}})
    generic_profile_checking_classes: list[str] | None = field(default=None,metadata={"label": "Generic profile checking classes", "allowed_in": {"alter","extract"}})
    raclist: list[str] | None = field(default=None,metadata={"label": "Raclisted classes", "allowed_in": {"alter","extract"}})
    global_access_classes: list[str] | None = field(default=None,metadata={"label": "Global access classes", "allowed_in": {"alter","extract"}})
    audit_classes: list[str] | None = field(default=None,metadata={"label": "Audit classes", "allowed_in": {"alter","extract"}})

def get_racf_options() -> dict[str, Any]:
    """Returns a dict with all of the RACF options"""
    if racfu_enabled:
        """Can be used to extract RACF options"""
        result = racfu({"operation": "extract", "admin_type": "racf-options"}) # type: ignore
        return result.result
    else:
        return {}
    
def get_active_classes() -> list[str]:
    """Returns a string list with all of the active classes on the system"""
    if racfu_enabled:
        """Returns a list of active classes on the system"""
        result = racfu({"operation": "extract", "admin_type": "racf-options"}) # type: ignore
        return result.result["profile"]["base"]["base:active_classes"] # type: ignore
    else:
        return []

def refresh_racf():
    """Refresh RACF"""
    if racfu_enabled:
        result = racfu(
            {
                "operation": "alter", 
                "admin_type": "racf-options", 
                "traits": {
                    "base:refresh": True,
                },
            },
        )
        return result.result["return_codes"]["racf_return_code"] # type: ignore

def update_racf_options(base: BaseSetroptsTraits):
    """Modify RACF options"""
    if racfu_enabled:
        traits = base.to_traits(prefix="base")
        
        result = racfu(
            {
                "operation": "alter", 
                "admin_type": "racf-options", 
                "traits":  traits,
            },
        )
        return result.result, result.result["return_codes"]["racf_return_code"] # type: ignore
    
def racf_change_rvary_password(password: str) -> int:
    """Change rvary password"""
    if racfu_enabled:
        result = racfu(
            {
                "operation": "alter", 
                "admin_type": "racf-options", 
                "traits": {
                    "base:rvary_switch_password": password.upper(),
                },
            },
        )
        return result.result["return_codes"]["racf_return_code"] # type: ignore
    else:
        return 8



from textual.app import ComposeResult
from textual.containers import HorizontalGroup
from textual.widgets import TabbedContent, TabPane

from blackwall.emoji import get_emoji
from blackwall.messages import OpenTab
from blackwall.panels.errors.history import PanelErrorHistory
from blackwall.panels.permits.resource_permit import PanelResourcePermit
from blackwall.settings import get_user_setting

from .panels.analysis.analysis import PanelAnalysis
from .panels.backout.backout import PanelBackout
from .panels.command_output.command_output import PanelCommandOutput
from .panels.dataset.dataset import PanelDataset
from .panels.group.group import PanelGroup
from .panels.resource.resource import PanelResource
from .panels.search.search import PanelSearch
from .panels.setropts.setropts import PanelSetropts
from .panels.users.user import PanelUser
from .panels.welcome.welcome import PanelWelcome

people_list = [
    "🧔",
    "🙋",
    "🙎",
    "🙍",
    "🙅",
    "🙆",
]

class TabSystem(HorizontalGroup):
    BINDINGS = [
        ("ctrl+u", "open_user", "Open user tab"),
        ("ctrl+g", "open_groups", "Open group profile tab"),
        ("ctrl+d", "open_dataset", "Open dataset profile tab"),
        ("ctrl+r", "open_resource", "Open resource profile tab"),
        ("ctrl+l", "open_command_output", "Open command output tab"),
        ("ctrl+f", "open_search", "Open search tab"),
        #("ctrl+a", "open_analysis", "Open analysis tab"),
        ("ctrl+o", "open_options", "Open RACF options tab"),
        ("ctrl+n", "open_resource_permits", "Open resource permits tab"),
        #("ctrl+b", "open_backout", "Open backout tab"),
        ("ctrl+w", "remove", "Remove active tab"),
        ("ctrl+shift+w", "clear", "Clear all tabs"),
    ]
    def __init__(self, *children, name = None, classes = None, disabled = False, markup = True):
        super().__init__(*children, name=name, classes=classes, disabled=disabled, markup=markup)
        self.tabs = TabbedContent(id="tab_system")

    def compose(self) -> ComposeResult:
        yield self.tabs

    def on_mount(self) -> None:
        default_tab = get_user_setting(section="tabs",setting="default_tab")
        if default_tab is not None:
            if default_tab == "user":
                self.post_message(OpenTab(f"{get_emoji(people_list)} User management",PanelUser()))
            elif default_tab == "group":
                self.post_message(OpenTab(f"{get_emoji("👥")} Group management",PanelGroup()))
            elif default_tab == "dataset":
                self.post_message(OpenTab(f"{get_emoji("📁")} Dataset profile mangement",PanelDataset()))
            elif default_tab == "resource":
                self.post_message(OpenTab(f"{get_emoji("☕")} Resource management",PanelResource()))
            elif default_tab == "commands":
                self.post_message(OpenTab(f"{get_emoji("📃")} Command history",PanelCommandOutput()))
            elif default_tab == "options":
                self.post_message(OpenTab("RACF options",PanelSetropts()))
            elif default_tab == "search":
                self.post_message(OpenTab(f"{get_emoji("🔎")} Search",PanelSearch()))
            else:
                self.post_message(OpenTab("Welcome!",PanelWelcome()))
        else:
            self.post_message(OpenTab("Welcome!",PanelWelcome()))

    async def on_open_tab(self, message: OpenTab):
        message.stop()
        tabs = self.get_child_by_id("tab_system",TabbedContent)
        new_tab = TabPane(message.title,message.content)
        await tabs.add_pane(new_tab)
        #Workaround, because switching tabs does not work when pressing a button I've had to disable the current tab and then re-enable it
        old_tab = tabs.active
        tabs.disable_tab(old_tab)
        def focus_tab():
            if new_tab.id is not None:
                tabs.active = new_tab.id
            tabs.enable_tab(old_tab)
        self.call_after_refresh(focus_tab)

    #Add new tab
    async def action_open_user(self) -> None:
        """Add a new user administration tab."""
        self.post_message(OpenTab(f"{get_emoji(people_list)} User management",PanelUser()))

    async def action_open_dataset(self) -> None:
        """Add a new dataset profile management tab."""
        self.post_message(OpenTab(f"{get_emoji("📁")} Dataset profile mangement",PanelDataset()))
    
    async def action_open_resource(self) -> None:
        """Add a new general resource profile management tab."""
        self.post_message(OpenTab(f"{get_emoji("☕")} Resource management",PanelResource()))
    
    async def action_open_groups(self) -> None:
        """Add a new group management tab."""
        self.post_message(OpenTab(f"{get_emoji("👥")} Group management",PanelGroup()))

    def action_open_search(self) -> None:
        """Add a new search tab."""
        self.post_message(OpenTab(f"{get_emoji("🔎")} Search",PanelSearch()))
    
    def action_open_analysis(self) -> None:
        """Add a new analysis tab."""
        self.post_message(OpenTab(f"{get_emoji("📊")} Health check",PanelAnalysis()))
    
    def action_open_command_output(self) -> None:
        """Add a new history tab."""
        self.post_message(OpenTab(f"{get_emoji("📃")} Command history",PanelCommandOutput()))

    def action_open_options(self) -> None:
        """Add a new RACF options tab."""
        self.post_message(OpenTab("RACF options",PanelSetropts()))

    def action_open_resource_permits(self) -> None:
        """Add a new resource permits tab."""
        self.post_message(OpenTab("Resource permits",PanelResourcePermit()))

    def action_open_error_log(self) -> None:
        """Add a new error log panel tab."""
        self.post_message(OpenTab("Error history",PanelErrorHistory()))

    def action_open_backout(self) -> None:
        """Add a new backout panel tab."""
        self.post_message(OpenTab(f"{get_emoji("↪")} Backout changes",PanelBackout()))

    #Remove current tab
    def action_remove(self) -> None:
        """Remove active tab."""
        tabs = self.get_child_by_id("tab_system",TabbedContent)
        active_pane = tabs.active_pane
        if active_pane is not None and active_pane.id is not None:
            tabs.remove_pane(active_pane.id)

    #Clear all tabs
    def action_clear(self) -> None:
        """Clear the tabs."""
        self.get_child_by_id("tab_system",TabbedContent).clear_panes()



from textual.app import ComposeResult
from textual.containers import Grid
from textual.screen import Screen
from textual.widgets import Button, Label

from blackwall.api.setropts import refresh_racf


class RefreshScreen(Screen):
    """Refresh RACF database screen"""

    BINDINGS = [("escape", "app.pop_screen", "Pop screen")]

    def compose(self) -> ComposeResult:
        yield Grid(
            Label("Are you sure you want to issue a refresh of the system?", id="question"),
            Button("Cancel", variant="primary", id="cancel", classes="modal-buttons"),
            Button("Confirm", variant="error", id="confirm", classes="modal-buttons"),
            id="dialog",
        )

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "confirm":
            return_code = refresh_racf()
            if return_code == 0:
                self.notify(f"Refresh successfully issued, return code: {return_code}",severity="information")
            else:
                self.notify(f"Refresh failed, return code: {return_code}",severity="error")
            self.app.pop_screen()
        else:
            self.app.pop_screen()




from textual.app import ComposeResult
from textual.containers import Grid
from textual.screen import Screen
from textual.widgets import Button, Input, Label

from blackwall.api.setropts import racf_change_rvary_password


class RvaryScreen(Screen):
    """Modal rvary password change screen"""

    BINDINGS = [("escape", "app.pop_screen", "Pop screen")]

    def compose(self) -> ComposeResult:
        yield Grid(
            Label("Change RVARY password for whole RACF database. Password must be up to 8 characters long. Allowed characters: alphanumeric and @#$", id="question"),
            Input(id="rvary_password",restrict=r"([a-zA-Z]*[0-9]*[\@\#\$]*)",placeholder="enter password...",max_length=8,password=True),
            Input(id="rvary_password_confirm",restrict=r"([a-zA-Z]*[0-9]*[\@\#\$]*)",placeholder="confirm password...",max_length=8,password=True),
            Button("Cancel", variant="primary", id="cancel", classes="modal-buttons"),
            Button("Confirm", variant="error", id="confirm", classes="modal-buttons"),
            id="dialog",
        )

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "confirm":
            input_value = self.query_exactly_one("#rvary_password",Input).value
            input_confirm_value = self.query_exactly_one("#rvary_password_confirm",Input).value
            if input_value == input_confirm_value:
                racf_change_rvary_password(input_value)
                self.app.pop_screen()
            else:
                self.notify("Password does not match",severity="error")
        else:
            self.app.pop_screen()



from dataclasses import dataclass

from textual.app import ComposeResult
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll
from textual.reactive import reactive
from textual.widgets import Button, Collapsible, Input, Label, RadioButton, Select

from blackwall.api import dataset
from blackwall.emoji import get_emoji
from blackwall.modals import generic_confirmation_modal
from blackwall.notifications import send_notification
from blackwall.panels.panel_mode import PanelMode
from blackwall.panels.traits_ui import get_traits_from_input, set_traits_in_input
from blackwall.regex import racf_id_regex


class PanelDatasetInfo(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Last change date:",classes="date-labels")
        yield Input(id="base_last_change_date",disabled=True,classes="date-fields")
        yield Label("Creation date:",classes="date-labels")
        yield Input(id="base_create_date",disabled=True,classes="date-fields")

class PanelDatasetName(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Profile name:")
        yield Input(id="profile_name")

class PanelDatasetOwner(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Owner:")
        yield Input(id="base_owner",max_length=8,restrict=racf_id_regex)

class PanelDatasetInstallationData(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Installation data:")
        yield Input(max_length=255,id="base_installation_data",classes="installation-data",tooltip="Installation data is an optional piece of data you can assign to a dataset profile. You can use installation data to describe whatever you want, such as owning department or what kind of data it protects")

class PanelDatasetAudit(VerticalGroup):
    def compose(self) -> ComposeResult:
        with Collapsible(title="Auditing"):
            yield Label("Notify user:")
            yield Input(id="base_notify_userid",max_length=8,restrict=racf_id_regex,classes="field-short-generic") 
            yield Label("Audit NONE:")
            yield Input(id="base_audit_none",classes="field-medium-generic")
            yield Label("Audit READ:")
            yield Input(id="base_audit_read",classes="field-medium-generic")
            yield Label("Audit UPDATE:")
            yield Input(id="base_audit_update",classes="field-medium-generic")
            yield Label("Audit CONTROL:")
            yield Input(id="base_audit_control",classes="field-medium-generic")
            yield Label("Audit ALTER:")
            yield Input(id="base_audit_alter",classes="field-medium-generic")
        
class PanelDatasetSecurityLevelAndCategories(VerticalGroup):
    def compose(self) -> ComposeResult:
        with Collapsible(title="Security level and category"):
            yield Label("Security level")
            yield Input(max_length=8,id="base_security_level",classes="field-short-generic")
            yield Label("Security category:")
            yield Input(max_length=8,id="base_security_category",classes="field-short-generic")
            yield Label("Security label:")
            yield Input(max_length=8,id="base_security_label",classes="field-short-generic")

class PanelDatasetUACC(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("UACC:")
        yield Select([("NONE", "NONE"),("READ", "READ"),("EXECUTE", "EXECUTE"),("UPDATE", "UPDATE"),("CONTROL", "CONTROL"),("ALTER", "ALTER")],value="NONE",classes="uacc-select",id="base_universal_access")

class PanelDatasetNotify(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Notify user:")
        yield Input(id="base_notify_userid",restrict=racf_id_regex,max_length=8,classes="field-short-generic")

class PanelDatasetVolume(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Input(id="base_volume")

class PanelDatasetSettings(VerticalGroup):
    def compose(self) -> ComposeResult:
        with Collapsible(title="General settings"):
            yield RadioButton(id="base_erase_data_sets_on_delete",label="Erase dataset on deletion")
            yield Label("Model profile volume:")
            yield Input(id="base_model_profile_volume",max_length=8,classes="field-short-generic")
            yield Label("Model profile class:")
            yield Input(id="base_model_profile_class",max_length=8,classes="field-short-generic")
            yield Label("Model profile generic:")
            yield Input(id="base_model_profile_generic",max_length=255,classes="field-long-generic")
            yield Label("Model profile:")
            yield Input(id="base_model_profile",max_length=255,classes="field-long-generic")
            yield Label("Dataset model profile:")
            yield Input(id="base_data_set_model_profile",max_length=255,classes="field-long-generic")
            yield Label("Volume:")
            yield Input(id="base_volume",max_length=8,classes="field-short-generic")

class PanelDatasetActionButtons(HorizontalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    delete_is_disabled = edit_mode is not True

    def __init__(self, save_action: str, delete_action: str):
        super().__init__()
        self.save_action = save_action
        self.delete_action = delete_action
    
    def compose(self) -> ComposeResult:
        yield Button("Create",id="save",action="save",classes="action-button")
        yield Button("Delete",action="delete",id="delete",variant="error",classes="action-button",disabled=self.delete_is_disabled)

    async def action_save(self):
        await self.app.run_action(self.save_action,default_namespace=self.parent)

    async def action_delete(self):
        await self.app.run_action(self.delete_action,default_namespace=self.parent)

@dataclass
class DatasetInfo:
    base_traits: dataset.BaseDatasetTraits | None = None
    mode: PanelMode = PanelMode.create

    profile_name: str = ""

class PanelDataset(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelDatasetInfo()
        yield PanelDatasetName()
        yield PanelDatasetOwner()
        yield PanelDatasetInstallationData()
        yield PanelDatasetUACC()
        yield PanelDatasetSettings()
        yield PanelDatasetSecurityLevelAndCategories()
        yield PanelDatasetAudit()
        yield PanelDatasetActionButtons(save_action="save_dataset_profile", delete_action="delete_dataset_profile")

    dataset_info: reactive[DatasetInfo] = reactive(DatasetInfo())

    def set_edit_mode(self):
        self.query_exactly_one("#profile_name",Input).disabled = True
        self.query_exactly_one("#delete",Button).disabled = False
        self.query_exactly_one("#save",Button).label = f"{get_emoji("💾")} Save"
        self.notify("Switched to edit mode",severity="information")

    def on_mount(self) -> None:
        if dataset.dataset_profile_exists(self.dataset_info.profile_name):
            self.query_exactly_one("#profile_name",Input).value = self.dataset_info.profile_name
            if self.dataset_info.base_traits is not None:
                set_traits_in_input(self,traits=self.dataset_info.base_traits,prefix="base")

            self.set_edit_mode()

    def action_delete_dataset_api(self) -> None:
        dataset_name = self.get_child_by_type(PanelDatasetName).get_child_by_id("profile_name",Input).value
        if dataset.dataset_profile_exists(dataset_name):
            message, return_code = dataset.delete_dataset_profile(dataset_name)
            
            if (return_code == 0):
                self.notify(f"Dataset profile {dataset_name} deleted, return code: {return_code}",severity="warning")
            else:
                self.notify(f"{message}, return code: {return_code}",severity="error")

    def action_delete_dataset_profile(self) -> None:
        dataset_name = self.get_child_by_type(PanelDatasetName).get_child_by_id("profile_name",Input).value
        generic_confirmation_modal(self,modal_text=f"Are you sure you want to delete dataset profile {dataset_name}?",confirm_action="delete_dataset_api",action_widget=self)

    def action_save_dataset_profile(self) -> None:
        dataset_name = self.get_child_by_type(PanelDatasetName).get_child_by_id("profile_name",Input).value
        dataset_profile_exists = dataset.dataset_profile_exists(dataset=dataset_name)

        operator = "alter" if dataset_profile_exists else "add"

        base_segment = get_traits_from_input(operator,self, prefix="base", trait_cls=dataset.BaseDatasetTraits)
        result = dataset.update_dataset_profile(
            dataset=dataset_name,
            create=not dataset_profile_exists,
            base=base_segment,
            )
        
        if not dataset_profile_exists:
            if (result == 0 or result == 4):
                self.notify(f"Dataset profile {dataset_name} created, return code: {result}",severity="information")
                #self.set_edit_mode()
            else:
                send_notification(self,message=f"Unable to create dataset profile, return code: {result}",severity="error")
        else:
            if result == 0:
                self.notify(f"Dataset profile {dataset_name} updated, return code: {result}",severity="information")
            else:
                send_notification(self,message=f"Unable to update dataset profile, return code: {result}",severity="error")



import subprocess
from datetime import datetime

from blackwall.settings import get_user_setting


def generate_command_meta_header(command_type: str,command: str) -> str:
    date_format = get_user_setting(section="locale",setting="date_format")

    now = datetime.now() # current date and time
    if date_format == "ymd":
        date_time = now.strftime("date: %Y/%m/%d time: %H:%M:%S")
    elif date_format == "mdy":
        date_time = now.strftime("date: %m/%d/%Y time: %H:%M:%S")
    else:
        date_time = now.strftime("date: %d/%m/%Y time: %H:%M:%S")
    
    local_now = now.astimezone()
    local_tz = local_now.tzinfo
    if local_tz is not None:
        local_tzname = local_tz.tzname(local_now)
        if local_tzname is not None:
            date_time = date_time + " " + local_tzname
    sep = "─" * 100
    return f"""
    ┌{sep}┐ 
     {command_type} command '{command}' 
     Executed on {date_time}
    └{sep}┘ 
    \n
    """

def execute_command(command: str) -> str | None:
    output = subprocess.run(f'tsocmd "{command}"', text=False, shell=True, check=True, capture_output=True)
    return generate_command_meta_header("TSO",command) + output.stdout.decode("utf-8", errors="ignore")



import importlib.util
from datetime import datetime

from textual.app import ComposeResult
from textual.containers import Container
from textual.widgets import Label

#Checks if textual_image can be imported
image_enabled = importlib.util.find_spec('textual_image')

if image_enabled:
    from textual_image.widget import SixelImage
else:
    print("##BLKWL_ERROR_3 Warning: could not find textual-image")    

#Checks if qrcode can be imported
qrcode_enabled = importlib.util.find_spec('qrcode')

if qrcode_enabled:
    import qrcode  # type: ignore
else:
    print("##BLKWL_ERROR_4 Warning: could not find qrcode")   

class AuditQRCode(Container):
    def compose(self) -> ComposeResult:
        now = datetime.now() # current date and time
        date_time = now.strftime("date: %m/%d/%Y time: %H:%M:%S")
        local_now = now.astimezone()
        local_tz = local_now.tzinfo
        local_tzname = local_tz.tzname(local_now)
        
        yield Label("Audit mode")
        if qrcode_enabled and image_enabled:
            img = qrcode.make(f"{date_time} {local_tzname}")
            pil_image = img.get_image()
            yield SixelImage(pil_image, classes="qrcode-image") # type: ignorez



from collections.abc import Generator
from dataclasses import Field, fields
from types import UnionType
from typing import get_args

from textual.widget import Widget
from textual.widgets import Collapsible, Input, Label, ListItem, ListView, RadioButton

from blackwall.api.traits_base import TraitsBase


def get_actual(field: Field) -> tuple[type,bool]:
    # UnionType is 'str | None'
    if isinstance(field.type, UnionType):
        # parse out actual type out of optional type
        # will be tuple (type(str), type(None))
        args = get_args(field.type)
        # the field is optional if type args contains 'type(None)'
        optional = type(None) in args
        # the actual type is the first non-'type(None)' in args
        actual_type = next((t for t in args if t is not type(None)), field.type)
    else:
        optional = False
        actual_type = field.type
    return actual_type, optional

def generate_trait_inputs(prefix: str, traits_class: type[TraitsBase],disabled: bool = False) -> Generator:
    for field in fields(traits_class):
        label = field.metadata.get("label")
        # only show an input field if it is labelled
        if label is not None:
            actual_type, optional = get_actual(field)

            input_args = field.metadata.get("input_args", {})

            input_id = f"{prefix}_{field.name}"

            if actual_type is str:
                yield Label(f"{label}{'*' if not optional else ''}:")
                yield Input(id=input_id, disabled=disabled, **input_args)
            elif actual_type is int:
                yield Label(f"{label}{'*' if not optional else ''}:")
                yield Input(id=input_id, type="integer", disabled=disabled, **input_args)
            elif actual_type == list[str]:
                with Collapsible(title=label,id=input_id):
                    yield ListView(disabled=disabled, **input_args)
            elif actual_type is bool:
                yield RadioButton(label=label, id=input_id, disabled=disabled, **input_args)

def generate_trait_section(title: str, prefix: str, traits_class: type[TraitsBase]) -> Generator:
    with Collapsible(title=title):
        yield from generate_trait_inputs(prefix=prefix,traits_class=traits_class)

def get_traits_from_input[T : TraitsBase](operator: str, widget: Widget, prefix: str, trait_cls: type[T]) -> T:
    value = trait_cls()
    for field in fields(trait_cls):
        actual_type, optional = get_actual(field)
        allowed_in = field.metadata.get("allowed_in")
        invalid_values = field.metadata.get("invalid_values")
        if allowed_in is not None and operator not in allowed_in:
            continue

        input_id = f"#{prefix}_{field.name}"
        label = field.metadata.get("label")
        if label is not None and actual_type != list[str]:
            field_value = widget.query_exactly_one(input_id).value
        else:
            field_value = None

        if actual_type is str:
            if field_value == "":
                field_value = None
        elif actual_type is int:
            if field_value == "" or field_value == 0 or field_value is None:
                field_value = None
            else:
                field_value = int(field_value)

        if invalid_values is not None and field_value in invalid_values: 
            field_value = None

        setattr(value, field.name, field_value)
    return value

def toggle_inputs(widget: Widget, prefix: str, traits: TraitsBase, disabled: bool):
    for field in fields(type(traits)):
        actual_type, optional = get_actual(field)
        label = field.metadata.get("label")
        # Only toggle a field if it has a label
        if label is not None:
            input_id = f"#{prefix}_{field.name}"
            if (actual_type is str or actual_type is int or actual_type is bool):
                widget.query_exactly_one(selector=input_id).disabled = disabled

def set_traits_in_input(widget: Widget, prefix: str, traits: TraitsBase):
    for field in fields(type(traits)):
        actual_type, optional = get_actual(field)
        label = field.metadata.get("label")
        # only show an input field if it is labelled
        if label is not None:
            input_id = f"{prefix}_{field.name}"
            field_value = getattr(traits,field.name)
            if (actual_type is str or actual_type is int):
                if field_value is not None:
                    widget.query_exactly_one(f"#{input_id}").value = str(field_value)
            elif actual_type is bool:
                if field_value is not None:
                    widget.query_exactly_one(f"#{input_id}", RadioButton).value = field_value
            elif actual_type == list[str]:
                collapsible_widget = widget.get_child_by_id(input_id,expect_type=Collapsible)
                list_widget = collapsible_widget.get_child_by_type(Collapsible.Contents).get_child_by_type(ListView)
                if field_value is not None:
                    for item in field_value:
                        list_widget.append(ListItem(Label(item)))



#API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

import importlib.util
from dataclasses import dataclass, field

from .traits_base import TraitsBase

#Checks if RACFU can be imported
racfu_enabled = importlib.util.find_spec('racfu')

if racfu_enabled:
    from racfu import racfu  # type: ignore
else:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")       

@dataclass
class BaseDatasetTraits(TraitsBase):
    #Add and alter fields
    owner: str | None = field(default=None, metadata={"label": "Owner"})
    audit_alter: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_control: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_none: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_read: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_read: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    audit_update: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    security_category: str | None = field(default=None,metadata={"label": "Security category", "allowed_in": {"alter"}})
    installation_data: str | None = field(default=None,metadata={"label": "Installation data", "allowed_in": {"add","alter"}})
    erase_data_sets_on_delete: bool | None = field(default=None,metadata={"label": "Erase datasets on deletion", "allowed_in": {"add","alter"}})
    model_profile_class: str | None = field(default=None,metadata={"label": "Model profile class", "allowed_in": {"add"}})
    model_profile_generic: str | None = field(default=None,metadata={"label": "Model profile generic", "allowed_in": {"add"}})
    tape_data_set_file_sequence_number: int | None = field(default=None,metadata={"allowed_in": {"add"}})
    model_profile: str | None = field(default=None,metadata={"label": "Model profile", "allowed_in": {"add"}})
    model_profile_volume: str | None = field(default=None,metadata={"label": "Model profile volume", "allowed_in": {"add"}})
    global_audit_alter: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_control: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_none: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_read: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    global_audit_update: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    level: int | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    data_set_model_profile: str | None = field(default=None,metadata={"allowed_in": {"add"}})
    notify_userid: str | None = field(default=None,metadata={"label": "Notify userid", "allowed_in": {"add","alter"}})
    security_label: str | None = field(default=None,metadata={"label": "Security label", "allowed_in": {"add","alter"}})
    security_level: str | None = field(default=None,metadata={"label": "Security level", "allowed_in": {"add","alter"}})
    racf_indicated_dataset: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    create_only_tape_vtoc_entry: str | None = field(default=None,metadata={"allowed_in": {"add"}})
    universal_access: str | None = field(default=None,metadata={"label": "UACC", "allowed_in": {"add","alter"}})
    data_set_allocation_unit: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})
    volume: str | None = field(default=None,metadata={"label": "Volume", "allowed_in": {"add","alter","extract"}})
    warn_on_insufficient_access: str | None = field(default=None,metadata={"allowed_in": {"add","alter"}})

    #Extraction fields
    access_list: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    access_count: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    access_type: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    access_id: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    alter_access_count: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    control_access_count: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    read_access_count: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    update_access_count: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    alter_volume: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    security_categories: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    create_date: str | None = field(default=None,metadata={"label": "Creation date", "allowed_in": {"extract"}})
    data_set_type: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    high_level_qualifier_is_group: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    creation_group_name: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    last_change_date: str | None = field(default=None,metadata={"label": "Change date","allowed_in": {"extract"}})
    auditing: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    global_auditing: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    use_tape_data_set_profile: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    resident_volume: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    resident_volumes: str | None = field(default=None,metadata={"allowed_in": {"extract"}})

@dataclass
class DFPDatasetTraits(TraitsBase):
    owner: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    ckds_data_key: str | None = field(default=None,metadata={"allowed_in": {"extract"}})

@dataclass
class TMEDatasetTraits(TraitsBase):
    roles: str | None = field(default=None,metadata={"allowed_in": {"extract"}})

#Dataset functions
def dataset_profile_exists(dataset: str) -> bool:
    """Checks if a dataset profile exists, returns true or false"""
    if racfu_enabled:
        result = racfu({"operation": "extract", "admin_type": "data-set", "profile_name": dataset.upper()})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return False

def get_dataset_profile(dataset: str) -> dict:
    """Doesn't handle dataset profiles that don't exist, recommend using dataset_profile_exists() first"""
    if racfu_enabled:
        result = racfu({"operation": "extract", "admin_type": "data-set", "profile_name": dataset.upper()})
        return result.result
    else:
        return {}

def update_dataset_profile(dataset: str, create: bool, base: BaseDatasetTraits):
    """Creates or updates a dataset profile"""
    if racfu_enabled:
        traits = base.to_traits(prefix="base")
        
        operation = "add" if create else "alter"
        
        result = racfu(
            {
                "operation": operation, 
                "admin_type": "data-set", 
                "profile_name": dataset.upper(),
                "traits":  traits,
            },
        )
        return result.result["return_codes"]["racf_return_code"]

def delete_dataset_profile(dataset: str) -> tuple[str, int]:
    """Deletes a dataset profile"""
    if racfu_enabled:
        result = racfu(
                {
                    "operation": "delete", 
                    "admin_type": "data-set", 
                    "profile_name": dataset.upper(),
                },
            )
        #TODO add error message
        return "", result.result["return_codes"]["racf_return_code"]
    else:
        return "RACFu can't be found", 8



from dataclasses import dataclass

from textual.app import ComposeResult
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll
from textual.reactive import reactive
from textual.widgets import Button, Collapsible, Input, Label, RadioButton, Select

from blackwall.api import resource
from blackwall.emoji import get_emoji
from blackwall.modals import generic_confirmation_modal
from blackwall.notifications import send_notification
from blackwall.panels.panel_mode import PanelMode
from blackwall.regex import racf_id_regex

from ..traits_ui import (
    generate_trait_section,
    get_traits_from_input,
    set_traits_in_input,
)


class PanelResourceInfo(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Creation date:",classes="date-labels")
        yield Input(id="base_create_date",disabled=True,classes="date-fields")
        yield Label("Last change date:",classes="date-labels")
        yield Input(id="base_last_change_date",disabled=True,classes="date-fields")
        yield Label("Last reference time:",classes="date-labels")
        yield Input(id="base_last_reference_date",disabled=True,classes="date-fields")

class PanelResourceNameAndClass(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Profile name:")
        yield Input(max_length=255,id="resource_profile_name",classes="resource-name-field")
        yield Label("Class:")
        yield Input(max_length=8,id="resource_profile_class",classes="class-field")
        yield Label("Owner:")
        yield Input(max_length=8,restrict=racf_id_regex,id="base_owner",classes="class-field")

class PanelResourceInstallationData(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Installation data:")
        yield Input(max_length=255,id="base_installation_data",classes="installation-data",tooltip="Installation data is an optional piece of data you can assign to a dataset profile. You can use installation data to describe whatever you want, such as owning department or what kind of data it protects")

class PanelResourceAccess(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("UACC:")
        yield Select([("NONE", "NONE"),("READ", "READ"),("EXECUTE", "EXECUTE"),("UPDATE", "UPDATE"),("CONTROL", "CONTROL"),("ALTER", "ALTER")],value="NONE",classes="uacc-select",id="base_universal_access",tooltip="It's advised that you keep this at NONE, UACC read or higher are unsecure, see z/OS RACF Administrator's Guide for more details")
        yield RadioButton(label="Warn on insufficient access",id="base_warn_on_insufficient_access",classes="generic-checkbox-medium")

class PanelResourceSegments(VerticalGroup):
    def compose(self) -> ComposeResult:
        with Collapsible(title="Resource profile segments"):
            yield from generate_trait_section(title="Started task data", prefix="stdata", traits_class=resource.STDATAResourceTraits)
            yield from generate_trait_section(title="ICSF", prefix="icsf", traits_class=resource.ICSFResourceTraits)
            yield from generate_trait_section(title="ICTX", prefix="ictx", traits_class=resource.ICTXResourceTraits)
            yield from generate_trait_section(title="JES", prefix="jes", traits_class=resource.JESResourceTraits)
            yield from generate_trait_section(title="Kerberos", prefix="kerb", traits_class=resource.KerbResourceTraits)
            yield from generate_trait_section(title="EIM", prefix="eim", traits_class=resource.EIMResourceTraits)
            yield from generate_trait_section(title="DLF data", prefix="dlfdata", traits_class=resource.DLFDataResourceTraits)
            yield from generate_trait_section(title="IDTPARMS", prefix="idtparms", traits_class=resource.IDTPARMSResourceTraits)
            yield from generate_trait_section(title="Session", prefix="session", traits_class=resource.SessionResourceTraits)
            yield from generate_trait_section(title="SVFMR", prefix="svfmr", traits_class=resource.SVFMRResourceTraits)
            yield from generate_trait_section(title="Proxy", prefix="proxy", traits_class=resource.ProxyResourceTraits)
            yield from generate_trait_section(title="MF policy", prefix="mfpolicy", traits_class=resource.MFPolicyResourceTraits)
            yield from generate_trait_section(title="SIGVER", prefix="sigver", traits_class=resource.SIGVERResourceTraits)
            yield from generate_trait_section(title="tme", prefix="tme", traits_class=resource.TMEResourceTraits)
            yield from generate_trait_section(title="SSIGNON", prefix="ssignon", traits_class=resource.SSIGNONResourceTraits)
            yield from generate_trait_section(title="Cfdef", prefix="Cfdef", traits_class=resource.CfdefResourceTraits)
            yield from generate_trait_section(title="Class Descriptor Table (CDT) info", prefix="cdtinfo", traits_class=resource.CDTINFOResourceTraits)

class PanelResourceActionButtons(HorizontalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    delete_is_disabled = edit_mode is not True

    def __init__(self, save_action: str, delete_action: str):
        super().__init__()
        self.save_action = save_action
        self.delete_action = delete_action
    
    def compose(self) -> ComposeResult:
        yield Button("Create",id="save",action="save",classes="action-button")
        yield Button("Delete",action="delete",id="delete",variant="error",classes="action-button",disabled=self.delete_is_disabled)

    async def action_save(self):
        await self.app.run_action(self.save_action,default_namespace=self.parent)

    async def action_delete(self):
        await self.app.run_action(self.delete_action,default_namespace=self.parent)

@dataclass
class ResourceInfo:
    base_traits: resource.BaseResourceTraits | None = None
    kerb_traits: resource.KerbResourceTraits | None = None
    dlf_traits: resource.DLFDataResourceTraits | None = None
    eim_traits: resource.EIMResourceTraits | None = None
    jes_traits: resource.JESResourceTraits | None = None
    icsf_traits: resource.ICSFResourceTraits | None = None
    ictx_traits: resource.ICTXResourceTraits | None = None
    idtparms_traits: resource.IDTPARMSResourceTraits | None = None
    session_traits: resource.SessionResourceTraits | None = None
    svfmr_traits: resource.SVFMRResourceTraits | None = None
    stdata_traits: resource.STDATAResourceTraits | None = None
    proxy_traits: resource.ProxyResourceTraits | None = None
    mfpolicy_traits: resource.MFPolicyResourceTraits | None = None
    sigver_traits: resource.SIGVERResourceTraits | None = None
    tme_traits: resource.TMEResourceTraits | None = None
    cdtinfo_traits: resource.CDTINFOResourceTraits | None = None
    ssignon_traits: resource.SSIGNONResourceTraits | None = None
    cfdef_traits: resource.CfdefResourceTraits | None = None
    mode: PanelMode = PanelMode.create

    resource_name: str = ""
    resource_class: str = ""

class PanelResource(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelResourceInfo()
        yield PanelResourceNameAndClass()
        yield PanelResourceInstallationData()
        yield PanelResourceAccess()
        yield PanelResourceSegments()
        yield PanelResourceActionButtons(save_action="save_resource_profile", delete_action="delete_resource_profile")

    resource_info: reactive[ResourceInfo] = reactive(ResourceInfo())

    def set_edit_mode(self):
        self.query_exactly_one("#resource_profile_name",Input).disabled = True
        self.query_exactly_one("#resource_profile_class",Input).disabled = True
        self.query_exactly_one("#delete",Button).disabled = False
        self.query_exactly_one("#save",Button).label = f"{get_emoji("💾")} Save"
        self.notify("Switched to edit mode",severity="information")

    def on_mount(self) -> None:
        if resource.resource_profile_exists(resource=self.resource_info.resource_name,resource_class=self.resource_info.resource_class):
            self.query_exactly_one("#resource_profile_name",Input).value = self.resource_info.resource_name
            self.query_exactly_one("#resource_profile_class",Input).value = self.resource_info.resource_class
            if self.resource_info.base_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.base_traits,prefix="base")
            
            if self.resource_info.kerb_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.kerb_traits,prefix="kerb")

            if self.resource_info.dlf_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.dlf_traits,prefix="dlf")

            if self.resource_info.eim_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.eim_traits,prefix="eim")

            if self.resource_info.jes_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.jes_traits,prefix="jes")

            if self.resource_info.icsf_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.icsf_traits,prefix="icsf")

            if self.resource_info.ictx_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.ictx_traits,prefix="ictx")

            if self.resource_info.idtparms_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.idtparms_traits,prefix="idtparms")

            if self.resource_info.session_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.session_traits,prefix="session")

            if self.resource_info.svfmr_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.svfmr_traits,prefix="svfmr")

            if self.resource_info.stdata_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.stdata_traits,prefix="stdata")

            if self.resource_info.proxy_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.proxy_traits,prefix="proxy")

            if self.resource_info.mfpolicy_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.mfpolicy_traits,prefix="mfpolicy")

            if self.resource_info.sigver_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.sigver_traits,prefix="sigver")

            if self.resource_info.tme_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.tme_traits,prefix="tme")
            
            if self.resource_info.cdtinfo_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.cdtinfo_traits,prefix="cdtinfo")

            if self.resource_info.ssignon_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.ssignon_traits,prefix="ssignon")

            if self.resource_info.cfdef_traits is not None:
                set_traits_in_input(self,traits=self.resource_info.cfdef_traits,prefix="cfdef")

            self.set_edit_mode()

    def action_delete_resource_profile_api(self):
        resource_profile_name = self.get_child_by_type(PanelResourceNameAndClass).get_child_by_id("resource_profile_name",Input).value
        resource_profile_class = self.get_child_by_type(PanelResourceNameAndClass).get_child_by_id("resource_profile_class",Input).value
        if resource.resource_profile_exists(resource_class=resource_profile_class,resource=resource_profile_name):
            message, return_code = resource.delete_resource_profile(resource_class=resource_profile_class,resource=resource_profile_name)
            
            if (return_code == 0):
                self.notify(f"Resource profile {resource_profile_name} deleted, return code: {return_code}",severity="warning")
            else:
                self.notify(f"{message}, return code: {return_code}",severity="error")

    def action_delete_resource_profile(self) -> None:
        resource_profile_name = self.get_child_by_type(PanelResourceNameAndClass).get_child_by_id("resource_profile_name",Input).value
        resource_profile_class = self.get_child_by_type(PanelResourceNameAndClass).get_child_by_id("resource_profile_class",Input).value
        generic_confirmation_modal(self,modal_text=f"Are you sure you want to delete resource profile {resource_profile_name} in {resource_profile_class}?",confirm_action="delete_resource_profile_api",action_widget=self)


    def action_save_resource_profile(self) -> None:
        resource_profile_name = self.get_child_by_type(PanelResourceNameAndClass).get_child_by_id("resource_profile_name",Input).value
        resource_profile_class = self.get_child_by_type(PanelResourceNameAndClass).get_child_by_id("resource_profile_class",Input).value
        resource_profile_exists = resource.resource_profile_exists(resource=resource_profile_name,resource_class=resource_profile_class)

        operator = "alter" if resource_profile_exists else "add"

        base_segment = get_traits_from_input(operator,self, prefix="base", trait_cls=resource.BaseResourceTraits)
        kerb_segment = get_traits_from_input(operator,self, prefix="kerb", trait_cls=resource.KerbResourceTraits)
        dlfdata_segment = get_traits_from_input(operator,self, prefix="dlfdata", trait_cls=resource.DLFDataResourceTraits)
        eim_segment = get_traits_from_input(operator,self, prefix="eim", trait_cls=resource.EIMResourceTraits)
        jes_segment = get_traits_from_input(operator,self, prefix="jes", trait_cls=resource.JESResourceTraits)
        icsf_segment = get_traits_from_input(operator,self, prefix="icsf", trait_cls=resource.ICSFResourceTraits)
        ictx_segment = get_traits_from_input(operator,self, prefix="ictx", trait_cls=resource.ICTXResourceTraits)
        idtparms_segment = get_traits_from_input(operator,self, prefix="idtparms", trait_cls=resource.IDTPARMSResourceTraits)
        session_segment = get_traits_from_input(operator,self, prefix="session", trait_cls=resource.SessionResourceTraits)
        svfmr_segment = get_traits_from_input(operator,self, prefix="svfmr", trait_cls=resource.SVFMRResourceTraits)
        stdata_segment = get_traits_from_input(operator,self, prefix="stdata", trait_cls=resource.STDATAResourceTraits)
        proxy_segment = get_traits_from_input(operator,self, prefix="proxy", trait_cls=resource.ProxyResourceTraits)
        mfpolicy_segment = get_traits_from_input(operator,self, prefix="mfpolicy", trait_cls=resource.MFPolicyResourceTraits)
        sigver_segment = get_traits_from_input(operator,self, prefix="sigver", trait_cls=resource.SIGVERResourceTraits)
        tme_segment = get_traits_from_input(operator,self, prefix="tme", trait_cls=resource.TMEResourceTraits)
        cdtinfo_segment = get_traits_from_input(operator,self, prefix="cdtinfo", trait_cls=resource.CDTINFOResourceTraits)
        ssignon_segment = get_traits_from_input(operator,self, prefix="ssignon", trait_cls=resource.SSIGNONResourceTraits)
        cfdef_segment = get_traits_from_input(operator,self, prefix="cfdef", trait_cls=resource.CfdefResourceTraits)

        result = resource.update_resource_profile(
            resource=resource_profile_name,
            resource_class=resource_profile_class,
            create=not resource_profile_exists,
            base=base_segment,
            kerb=kerb_segment,
            cdtinfo=cdtinfo_segment,
            dlfdata=dlfdata_segment,
            eim=eim_segment,
            jes=jes_segment,
            icsf=icsf_segment,
            ictx=ictx_segment,
            idtparms=idtparms_segment,
            session=session_segment,
            svfmr=svfmr_segment,
            stdata=stdata_segment,
            proxy=proxy_segment,
            mfpolicy=mfpolicy_segment,
            sigver=sigver_segment,
            tme=tme_segment,
            ssignon=ssignon_segment,
            cfdef=cfdef_segment,
            )
        
        if not resource_profile_exists:
            if (result == 0 or result == 4):
                self.notify(f"General resource profile {resource_profile_name} created, return code: {result}",severity="information")
                #self.set_edit_mode()
            else:
                send_notification(self,message=f"Unable to create general resource profile, return code: {result}",severity="error")
        else:
            if result == 0:
                self.notify(f"General resource profile {resource_profile_name} updated, return code: {result}",severity="information")
            else:
                send_notification(self,message=f"Unable to update general resource profile, return code: {result}",severity="error")



from textual.theme import Theme

ibm_3270_theme = Theme(
    name="IBM 3270",
    primary="#00feff",
    secondary="#17f9e1",
    accent="#ff0000",
    foreground="#f3ebdb",
    background="#000000",
    success="#0efd03",
    warning="#f5ff30",
    error="#ff0000",
    surface="#000000",
    panel="#0efd03",
    dark=True,
    variables={
        "block-cursor-text-style": "none",
        "footer-key-foreground": "#88C0D0",
        "input-selection-background": "#81a1c1 35%",
        "border": "#17f9e1",
        "border-blurred": "#17f9e1",
        "footer-foreground": "#000000",
        "footer-description-foreground": "#000000",
    },
)


#Project by Emma Skovgaard

#Enables true colors to be active by default
from os import environ

if "TEXTUAL_COLOR_SYSTEM" not in environ:
    environ["TEXTUAL_COLOR_SYSTEM"] = "truecolor"

import importlib.util

from .app import Blackwall

spec = importlib.util.find_spec('textual_image')

if spec:
    import textual_image.renderable  # noqa: F401
else:
    print("##BLKWL_ERROR_3 Warning: could not find textual-image")    

def main():
    Blackwall().run()

if __name__ == "__main__":
    main()




from textual.app import ComposeResult
from textual.containers import Grid
from textual.screen import Screen
from textual.widgets import Button, Label


class WarningScreen(Screen):
    """Warning screen"""

    BINDINGS = [("escape", "app.pop_screen", "Pop screen")]

    def __init__(self, dialog_text: str):
        super().__init__()
        self.dialog_text = dialog_text

    def compose(self) -> ComposeResult:
        yield Grid(
            Label(self.dialog_text, id="question"),
            Button("Dismiss", variant="warning", id="dismiss", classes="single-modal-button"),
            id="dialog",
        )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.app.pop_screen()